{
  Cyrille de Brebisson utility classes
  This unit is written by Cyrille de Brebisson

  Changes marked with WGG are copyright 2002, William G. Graves

}
Unit CDBShUtils;

Interface
Uses classes, forms, controls, math, windows, graphics, sysutils, messages,     // wgg removed dib
     extctrls, stdctrls, dialogs, ShellApi;
Const                          
  HexStr = '0123456789ABCDEF';

Type
  THPStream = Class(TStream)
  Private
    Data: Array Of byte;
    FPosition: longint;
  Protected
    Procedure SetSize(NewSize: Longint); Override;
  Public
    Constructor Create;
    Function Read(Var Buffer; Count: Longint): Longint; Override;
    Function Write(Const Buffer; Count: Longint): Longint; Override;
    Function Seek(Offset: Longint; Origin: Word): Longint; Override;
    Function WriteInt(i: int64; count: integer): longint; Overload;
    Function WriteInt(i: integer): longint; Overload;
    Procedure InsertInt(i: integer; pos: integer);
    Procedure PokeInt(i: int64; pos: integer; count: integer); Overload;
    Procedure PokeInt(i: integer; pos: integer); Overload;
    Function readInt: integer; Overload;
    Function ReadInt(n: integer): int64; Overload;
    Procedure SaveToHPFile(f: TFileName; HPHP:string='HPHP48-X');              // wgg added HPHP
//    Function WriteGrob(p: TDib8; gray: boolean = false): integer;
    Procedure LoadFromFile(f: TFileName);
    Function WriteName(s: String): integer; // Write SIZE, string, SIZE
    Function WriteStringObj(s: String): integer; // Write a string object from a PC String
    Function WriteString(s: String): integer; // Write all chr of the string
    Function PData: PByte;
    Procedure LoadFromByteString(const s: string);
    Function GetByteString: string;
  End;

  TCustomPanel2 = Class(TScrollBox)
  Protected
    FCanvas: TCanvas;
    FOnPaint: TNotifyEvent;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    Procedure Paint; Virtual;
    Procedure PaintWindow(DC: HDC); Override;
    Function GetCanvas: TCanvas;
    Procedure Resize; Override;
    Property Color Default clBtnFace;
    Property ParentColor Default False;
    Property OnPaint: TNotifyEvent Read FOnPaint Write FOnPaint;
  Public
    Constructor Create(AOwner: TComponent); Override;
    Destructor Destroy; Override;
    Property Canvas: TCanvas Read GetCanvas;
  End;

  TVersionLabel = class(TLabel)
  Public
  Private
    FCaption: string;
    procedure SetCaption(const Value: string);
  Published
    Property Caption: string read FCaption write SetCaption;
  end;

  TOnHexaWrite = procedure(Sender: TObject; Const adr: integer; Const Value: byte; Const Nibble: boolean) of object;
  TOnHexaRead = function(Sender: TObject; const adr: integer): byte of object;
  THexaView = class(TPanel)
  protected
  private
    FNibbles: boolean;
    FReadOnly: boolean;
    FCurrentTop: integer;
    FNbBytePerLine: integer;
    FMaxAdr: integer;
    FMinAdr: integer;
    FCurrentEdit: integer;
    FAdrWidth: integer;
    FOnRead: TOnHexaRead;
    FOnWrite: TOnHexaWrite;
    FHexaEdit: boolean;
    FPaint: Boolean;
    FWidth, FHeight: integer;
    NbLines: integer;
    ed: TEdit;
//    FSc: TScrollBar;
    procedure SetAdrWidth(const Value: integer);
    procedure SetCurrentEdit(const Value: integer);
    procedure SetCurrentTop(const Value: integer);
    procedure SetMaxAdr(const Value: integer);
    procedure SetMinAdr(const Value: integer);
    procedure SetNbBytePerLine(const Value: integer);
    procedure SetNibbles(const Value: boolean);
    procedure SetOnRead(const Value: TOnHexaRead);
    procedure SetOnWrite(const Value: TOnHexaWrite);
    procedure SetReadOnly(const Value: boolean);
    procedure SetHexaEdit(const Value: boolean);
    Procedure EdKeyPress(Sender: TObject; var Key: char);
    Procedure EdKeyDown(Sender: TObject; var key: word; Shift: TShiftState);
  public
    Procedure Paint; override;
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  published
    property NbBytePerLine: integer read FNbBytePerLine write SetNbBytePerLine;
    Property Nibbles: boolean read FNibbles write SetNibbles;
    Property MinAdr: integer read FMinAdr write SetMinAdr;
    Property MaxAdr: integer read FMaxAdr write SetMaxAdr;
    Property CurrentTop: integer read FCurrentTop write SetCurrentTop;
    Property CurrentEdit: integer read FCurrentEdit write SetCurrentEdit;
    Property ReadOnly: boolean read FReadOnly write SetReadOnly;
    Property AdrWidth: integer read FAdrWidth write SetAdrWidth;
    Property OnWrite: TOnHexaWrite read FOnWrite write SetOnWrite;
    Property OnRead: TOnHexaRead read FOnRead write SetOnRead;
    Property HexaEdit: boolean read FHexaEdit write SetHexaEdit;
  end;

  TEditLabel = class (TCustomLabel)
  private
    FEditZone: string;
    FEditColor: TColor;
    FEditPos: integer;
    ed: TEdit;
    FKeyDown: TKeyEvent;
    FKeyPress: TKeyPressEvent;
    FHexOnly: boolean;
    fDecOnly: boolean;
    FSpinEdit: boolean;
    Fmin: integer;
    Fmax: integer;
    FOnChange: TNotifyEvent;
    FAfterKeyPress: TKeyPressEvent;
    procedure SetEditColor(const Value: TColor);
    procedure SetEditZone(const Value: string);
    procedure SetEditPos(const Value: integer);
    function ReadValue: int64;
    procedure WriteValue(const Value: int64);
    procedure SetSpinEdit(const Value: boolean);
    procedure Setmax(const Value: integer);
    procedure Setmin(const Value: integer);
  Protected
    poses: Array of TRect;
    SpinButs: array [0..1] of TRect;
    procedure DoDrawText(var Rect: TRect; Flags: Longint); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); Override;
    Procedure EdKeyPress(Sender: TObject; var Key: char);
    Procedure EdKeyDown(Sender: TObject; var key: word; Shift: TShiftState);
    Procedure EdExit(Sender: TObject);
    Procedure checkValue;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property EditZone: string read FEditZone write SetEditZone;
    Property EditColor: TColor read FEditColor write SetEditColor;
    property EditPos: integer read FEditPos write SetEditPos;
    Property SpinEdit: boolean read FSpinEdit write SetSpinEdit;
    Property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    Property minValue: integer read Fmin write Setmin default 0;
    Property maxValue: integer read Fmax write Setmax default 100;
    Property Value: int64 read ReadValue write WriteValue stored false;
    Property HexOnly: boolean read FHexOnly write FHexOnly;
    property DecOnly: boolean read fDecOnly write fDecOnly;
    Property OnKeyDown: TKeyEvent read FKeyDown write FKeyDown;
    Property OnKeyPress: TKeyPressEvent read FKeyPress write FKeyPress;
    Property OnAfterKeyPress: TKeyPressEvent read FAfterKeyPress write FAfterKeyPress;
  end;

  TReadProc = function(Sender: TObject): string of object;
  TWriteProc = procedure(Sender: TObject; const Value: string) of object;
  TCreateProcess = class(TComponent)
  private
    h: THandle;
    FCommandLine: String;
    FFileName: TFileName;
    FWait: boolean;
    FExitCode: DWORD;
    FRun: Boolean;
    FRedirectIO: boolean;
    FOnRead: TReadProc;
    FOnError: TWriteProc;
    FOnWrite: TWriteProc;
    StdIn1, StdIn2, StdOut1, StdOut2, StdErr1, StdErr2: THandle;
    FHideChild: Boolean;
    FRunningdir: string;
    wthread: TThread;
    procedure SetRun(const Value: boolean);
  Public
    Procedure WaitForCompletion;
    destructor Destroy; override;
    Procedure Write(const Value: String);
  Published
    Property FileName: TFileName read FFileName write FFileName;
    Property CommandLine: String read FCommandLine write FCommandLine;
    Property Run: boolean read FRun write SetRun;
    Property WaitUntiFinish: boolean read FWait write FWait;
    Property ExitCode: DWORD read FExitCode;
    Property Handle: THandle read h;
    Property RedirectIO: boolean read FRedirectIO write FRedirectIO;
    Property OnRead: TReadProc read FOnRead write FOnRead;
    Property OnWrite: TWriteProc read FOnWrite write FOnWrite;
    Property OnError: TWriteProc read FOnError write FOnError;
    Property HideChild: Boolean read FHideChild write FHideChild;
    Property RunningDir: string read FRunningDir write FRunningdir;
  end;

  T38Object = class
  public
    Typ: char;
    PCName: string;
    HPName: string;
  end;

  THP38Dir = class
  private
    List: TList;
//    FCount: integer;
    function GetItem(i: integer): T38Object;
    function GetCount: integer;
    Procedure EmptyList;
  public
    constructor create;
    destructor Destroy; override;
    Procedure LoadFromFile(f: TFileName);
    Procedure SaveToFile(f:TFileName; h: String);
    property Items[i: integer]: T38Object read GetItem; default;
  published
    property Count: integer read GetCount;
  end;

  function GetLongName(s: string): string;                                     // wgg
  function crScan(s: string): boolean;                                         // wgg
  function crRemove(s: string): string;                                        // wgg
  Function GetTempDir: string;
  Function GetFileVersion: String;
  Function SendThroughRS232(Com: String; Data: String): boolean;
  Function Pos2(Const SubStr, S: String; InitialPos: integer): integer;
  Function RemoveTab(s: String; TabSize: integer = 2): String;
  Function SkipWhite(Const s: String; i: integer): integer;
  Function FindWhite(Const s: String; i: integer): integer;
  Function GetToken(Const s: String; i: integer): String;
  Function NxtToken(Const s: String; i: integer): integer;
  Function FileToString(f: TFileName): String; overload;
  Function FileToString(f: TFileName; Readsize: integer): String; overload;
  Function HPFileToString(f: TFileName): string;
  Procedure StringToFile(s: String; f: TFileName);
  Function NibbleToByte(s: String): String;
  Function ByteToNibble(s: String): String;
  Function ByteToNibble2(s: String): String;
  Function IntToHex2(i: int64; digit: integer): String;
  Function HexToStr(s: String): int64;
  Procedure swap(var i, j: integer); overload;
  procedure swap(var i, j: byte); overload;
  procedure swap(var i, j: char); overload;
  Procedure RevString(var s: string);
  Procedure Repl(Var Dest: String; Source: String; pos: integer);
  Function Removeslashslash(s: string): string;
  Function ExpandText(const s: string): string;
  Function ShrinkText(const s: string): string;
  Procedure Browse(E: TEdit; t: string; s: string);                         // wgg
  Function ApplicationPath: string;
  Procedure CopyFile(SourceFile, DestFile: TFileName);
  function stringreplace(s1, sf, sr: string): string;
  function StringCrc(Const S: string): integer;
  Function StrToHex(s: string): integer;
  function GetSpaces(number: integer): string;
  Function TokenAtPos(s: string; var p: integer): string;
  Function Distance2(x1, y1, x2, y2: single): real;
  Function AddSpaces(const s: string; int: integer): string;
  Function BinToHex(const s: string): string;
  Function BinToHexHP(const s: string): string;
  Function HexToBinHP(const s: string): string;
  Function cutstr(const s: string; size: integer): string;
  function QuoteIf(s: string): string;                                    // wgg
  function BackToName(s: string): integer;                                // wgg

procedure Register;

Implementation

procedure Register;
begin
  RegisterComponents('CDBShort', [THexaView, TVersionLabel, TCreateProcess, TEditLabel]);  // wgg was exemples
end;



// wgg returns a long file name given a short
function GetLongName(s: string): string;
var
  ShFileInfo: TSHFileInfo;
begin
  SHGetFileInfo(PChar(s), 0, ShFileInfo, SizeOf(ShFileInfo), SHGFI_DISPLAYNAME);
  result:= ShFileInfo.szDisplayName;
end;

{// wgg another method of getting long names
function ShortLongName(f: TFileName): TFileName;
var
  fnd: TWIN32FindData;
  Handle: THandle;
begin
  result:= f;
  Handle := FindFirstFile(PChar(f), fnd);
  if Handle <> ERROR_INVALID_HANDLE then
    if String(fnd.cFileName) <> '' then
      result:= copy(f, 1, pos(ExtractFileName(f), f)-1)+TFileName(fnd.cFileName) ;
  Windows.FindClose(Handle);
end; }

// wgg work backwards in a string, until a space.
// then continue until a non-space and return length of the left hand side
// Used for * File xxx error msgs in a long name environment
function BackToName(s: string): integer;
var
  i, j: integer;
begin
  j:= length(s);
  for i:=j downto 1 do
    if s[i] = ' ' then break;
  if i>1 then
    for j:=i-1 downto 1 do
      if s[j]<>' ' then break;
  result:= j;
end;

// wgg add quotes around a string if it has any embedded spaces
// used for INCLUDE file names
function QuoteIf(s: string): string;
begin
  if Pos(' ', s)<> 0 then result:= '"'+s+'"'
  else result:= s;
end;

// wgg function for removing cr #13 from a file string
// This keeps the Unix people happy
function crRemove(s: string): string;                                        // wgg
var
  i, j: integer;
begin
  i:= 1;
  j:= 1;
  setLength(result, length(s));        // so we dont keep stretching a character at a time
  while i <= length(s) do
  begin
    result[j]:= s[i];
    if s[i] <> #13 then inc(j);
    inc(i);
  end;
  setLength(result, j-1);
end;

// wgg function scans for CR #13 in a file
function crScan(s: string): boolean;                                        // wgg
var
  i: integer;
begin
  i:= 1;
  result:= false;
  while (i <= length(s)) do
  begin
    if s[i] = #13 then
      begin
        result:=true;
        break;
      end;
    if s[i] = #10 then break;              // we should see #13 before #10
    inc(i);
  end;
end;

Function cutstr(const s: string; size: integer): string;
var
  i: integer;
Begin
  result:= '';
  i:=1;
  while i<=length(s) do
  begin
    result:= result+copy(s, i, size)+#13#10;
    inc(i, size);
  end;
end;

const
  AsciiBin = '0123456789ABCDEF';
Function BinToHexHP(const s: string): string;
var
  i: integer;
Begin
  setlength(result, 2*Length(s));
  for i:=0 to length(s)-1 do
  Begin
    result[2*i+1]:= AsciiBin[ord(s[i+1]) and $F+1];
    result[2*i+2]:= AsciiBin[(ord(s[i+1]) div 16) and $F+1];
  end;
end;

Function HexToBinHP(const s: string): string;
var
  i: integer;
Begin
  setlength(result, (Length(s)+1) div 2);
  for i:=0 to length(result)-1 do
    result[i+1]:= chr((pos(s[i*2+2], HexStr)-1)*16+(pos(s[i*2+1], HexStr)-1));
end;

Function BinToHex(const s: string): string;
var
  i: integer;
Begin
  setlength(result, 2*Length(s));
  for i:=0 to length(s)-1 do
  Begin
    result[2*i+2]:= AsciiBin[ord(s[i+1]) and $F+1];
    result[2*i+1]:= AsciiBin[(ord(s[i+1]) div 16) and $F+1];
  end;
end;


Function AddSpaces(const s: string; int: integer): string;
Begin
  result:= s;
  while length(result)<int do result:= result+' ';
end;

Function Distance2(x1, y1, x2, y2: single): real;
begin
  result:= sqr(x1-x2)+sqr(y1-y2);
end;

Function TokenAtPos(s: string; var p: integer): string;
var
  i: integer;
Begin
  i:= skipwhite(s, p);
  p:= findwhite(s, i);
  result:= copy(s, i, p-i);
end;

function GetSpaces(number: integer): string;
begin
  SetLength(result, number);
  FillMemory(@(result[1]), number, 32);
end;

type
  TMatrix=array [0..15, 0..15] of integer;
  TArray=array[0..$100-1] of integer;
var
  CrcTable:TMatrix;
  CrcTableArray:TArray absolute CrcTable;

Function StrToHex(s: string): integer;
var
  i: integer;
Begin
  val('$'+s, result, i);
  if i<>0 then
    raise exception.create(s+' is not an hex number');
end;

function StringCrc(Const S: string): integer;
var
  i,l,j,k :integer;
  c:^Byte;
begin
  result:=0;
  l:=length(s);
  c:=@s[1];
  for i:=1 to l do
  begin
    j:=c^;
    inc(c);
    k:=(result and $F) shl 4;
    result:= (result shr 4) xor CrcTAbleArray[k+ (j and $F)];
    k:=(result and $F) shl 4;
    result:= (result shr 4) xor CrcTAbleArray[k+ (j shr 4)];
  end;
end;

procedure CrcInitTable;
var
  crc, input: integer;
Begin
  for crc:= 0 to 15 do
    for input:= 0 to 15 do
      CrcTable[crc, input]:= (crc xor input)*$1081
end;

function stringreplace(s1, sf, sr: string): string;
var
  i: integer;
begin
  result:= '';
  i:= pos(sf, s1);
  while i<>0 do
  begin
    result:= result+copy(s1, 1, i-1)+sr;
    s1:= copy(s1, i+length(sf), maxint);
    i:= pos(sf, s1);
  end;
  result:= result+s1;
end;

Procedure CopyFile(SourceFile, DestFile: TFileName);
Begin
  StringToFile(FileToString(SourceFile), DestFile);
end;

Function ApplicationPath: string;
Begin
  result:= ExtractFileDir(Application.ExeName);
end;

Procedure Browse(E: TEdit; t: string; s: string);           // wgg changed to be a filter string, added title
Begin
  with TOpenDialog.Create(nil) do
    try
      InitialDir:= GetCurrentDir;                             // wgg
      Title:= t;
      Filter:= s;
      if execute then
        e.text:= FileName;
    finally
      free;
    end;
end;

Function ExpandText(const s: string): string;
var
  i: integer;
Begin
  SetLength(result, length(s));
  for i:= 1 to length(s) do
    case s[i] of
      #10: result[i]:= chr(256-10);
      #13: result[i]:= chr(256-13);
      else result[i]:= s[i];
    end;
end;

Function ShrinkText(const s: string): string;
var
  i: integer;
Begin
  SetLength(result, length(s));
  for i:= 1 to length(s) do
    case s[i] of
      chr(256-10): result[i]:= #10;
      chr(256-13): result[i]:= #13;
      else result[i]:= s[i];
    end;
end;

Function Removeslashslash(s: string): string;
Begin
  result:= '';
  while s<>'' do
    if (s[1]='\') then
    begin
      result:=result+'\';
      while (s<>'') and (s[1]='\') do
        s:= copy(s, 2, maxint);
    end else begin
      result:= result+s[1];
      s:= copy(s, 2, maxint);
    end;
end;

Function GetTempDir: string;
var
  b: array [0..1000] of char;
Begin
  GetTempPath(1000, @(b[0]));
  result:= strPas(b);
  if result[length(result)]<>'\' then result:= result+'\';      // wgg
end;

Function HPFileToString(f: TFileName): string;
Begin
  result:= FileToString(f);
  if copy(result, 1, 5)='HPHP4' then
    result:= copy(result, 9, maxint);
  result:= ByteToNibble(result);
end;

Procedure RevString(var s: string);
var
  i: integer;
  l: integer;
Begin
  l:= length(s);
  for i:= 0 to (l-1) div 2 do
    swap(s[i+1], s[l-i]);
end;

Procedure swap(var i, j: integer);
var
  t: integer;
begin
  t:= i;
  i:= j;
  j:= t;
end;

procedure swap(var i, j: byte);
var
  t: byte;
begin
  t:= i;
  i:= j;
  j:= t;
end;

procedure swap(var i, j: char);
var
  t: char;
begin
  t:= i;
  i:= j;
  j:= t;
end;

Function IntToHex2(i: int64; digit: integer): String;
Var
  j: integer;
Begin
  SetLength(Result, digit);
  For j := 0 To digit - 1 Do
  Begin
    result[digit - j] := HexStr[i And $F + 1];
    i := i Div 16;
  End;
End;

Function GetToken(Const s: String; i: integer): String;
Var
  j: integer;
Begin
  j := SkipWhite(s, 1);
  While i > 1 Do
  Begin
    j := SkipWhite(s, FindWhite(s, j));
    dec(i);
  end;
  i := FindWhite(s, j);
  result := copy(s, j, i - j);
End;

Function HexToStr(s: String): int64;
Var
  p: pchar;
  i, l: integer;
Begin
  result := 0;
  l := length(s);
  If l <> 0 Then
  Begin
    p := pointer(s);
    For i := 1 To l Do
    Begin
      result := result * 16;
      If p^ < '0' Then
        Raise exception.Create(s + ' is Not an hex value');
      If p^ <= '9' Then
        result := result + ord(p^) - ord('0')
      Else
      Begin
        If p^ < 'A' Then
          Raise exception.Create(s + ' is Not an hex value');
        If p^ <= 'F' Then
          result := result + ord(p^) - ord('A') + 10
        Else
        Begin
          If p^ < 'a' Then
            Raise exception.Create(s + ' is Not an hex value');
          If p^ <= 'f' Then
            result := result + ord(p^) - ord('a') + 10
          Else
            Raise exception.Create(s + ' is Not an hex value');
        End;
      End;
      inc(p);
    End;
  End;
End;

Procedure Repl(Var Dest: String; Source: String; pos: integer);
Begin
  If Pos + length(Source) > Length(Dest) Then
    SetLength(Dest, Pos + length(Source));
  move(pchar(source)^, dest[pos], length(source));
End;

Procedure StringToFile(s: String; f: TFileName);
Var
  Fh: TextFile;
Begin
  try
    AssignFile(Fh, f);
    Rewrite(Fh);
    Write(Fh, s);
  finally
    CloseFile(Fh);
 { With TFileStream.Create(f, fmCreate) Do    // wgg was failing for extprg
  Try                                         // and I don't know why
    Write(pchar(s)^, length(s));
  Finally
    free; }
  End;  
End;

Function FileToString(f: TFileName; Readsize: integer): String;
Begin
  With TFileStream.Create(f, fmOpenRead) Do
  Try
    SetLength(result, max(size,Readsize));
    read(pchar(result)^, max(size,Readsize));
  Finally
    free;
  End;
End;

Function FileToString(f: TFileName): String;
Begin
  With TFileStream.Create(f, fmOpenRead) Do
  Try
    SetLength(result, size);
    read(pchar(result)^, size);
  Finally
    free;
  End;
End;

Function NibbleToByte(s: String): String;
Var
  i: integer;
Begin
  SetLength(result, (length(s) + 1) Div 2);
  For i := 0 To length(result) - 1 Do
    result[i + 1] := chr((ord(s[i * 2 + 1]) And $F) + ((ord(s[i * 2 + 2]) And $F) * 16));
End;

Function ByteToNibble(s: String): String;
Var
  i: integer;
Begin
  SetLength(result, length(s) * 2);
  For i := 0 To length(s) - 1 Do
  Begin
    result[i * 2 + 1] := chr(ord(s[i + 1]) And $F);
    result[i * 2 + 2] := chr((ord(s[i + 1]) Div 16) And $F);
  End;
End;

Function ByteToNibble2(s: String): String;
Var
  i: integer;
Begin
  SetLength(result, length(s) * 2);
  For i := 0 To length(s) - 1 Do
  Begin
    result[i * 2 + 1] := chr((ord(s[i + 1]) Div 16) And $F);
    result[i * 2 + 2] := chr(ord(s[i + 1]) And $F);
  End;
End;

Function SkipWhite(Const s: String; i: integer): integer;
Begin
  While (i <= length(s)) And (s[i] <= ' ') Do
    inc(i);
  result := i;
End;

Function FindWhite(Const s: String; i: integer): integer;
Begin
  While (i <= length(s)) And (s[i] > ' ') Do
    inc(i);
  result := i;
End;

Function NxtToken(Const s: String; i: integer): integer;
Begin
  result := SkipWhite(s, FindWhite(s, i));
End;


Function RemoveTAb(s: String; TabSize: integer = 2): String;
Var
  i, j, d: integer;
Begin
  i := 0;
  d := 0;
  For j := 1 To Length(s) Do
  Begin
    If Length(result) < I + TabSize + 1 Then
      SetLength(Result, I + TabSize + 10000);
    Case s[j] Of
      #9:
        Begin
          inc(i);
          result[i] := ' ';
          While (i - d) Mod TabSize <> 0 Do
          Begin
            inc(i);
            result[i] := ' ';
          End;
        End;
      #10:
        Begin
          inc(i);
          result[i] := s[j];
         d := i;    
        End;
    Else
      Begin
        inc(i);
        result[i] := s[j];
      End;
    End;
  End;
  SetLength(result, i);
End;

Function Pos2(Const SubStr, S: String; InitialPos: integer): integer;
Var
  substrlenght: integer;

  Function StrEqual(p: integer): Boolean;
  Var
    i: integer;
  Begin
    For i := 1 To substrlenght Do
      If SubStr[i] <> s[i - 1 + p] Then
      Begin
        result := False;
        exit;
      End;
    result := true;
  End;

Begin
  substrlenght := Length(SubStr);
  If substrlenght = 0 Then
    result := 0
  Else
  Begin
    result := InitialPos;
    While (result <= length(s) - length(SubStr) + 1) And Not StrEqual(result) Do
      inc(result);
    If (result > length(s) - length(SubStr) + 1) Then
      result := 0;
  End;
End;

{ THPStream }

Constructor THPStream.Create;
Begin
  Inherited create;
  fposition := 0;
End;

Procedure THPStream.LoadFromFile(f: TFileName);
Var
  i: integer;
  bf: String;
Begin
  bf:= HPFileToString(f);
  size := Length(bf);
  For i := 1 To Length(bf) Do
    self.WriteInt(ord(bf[i]), 1);
  seek(0, soFromBeginning);
End;

Procedure THPStream.InsertInt(i: integer; pos: integer);
var
  j: integer;
Begin
  SetLength(data, length(data)+5);
  for j:= length(data)-6 downto pos  do
    data[j+5]:= data[j];
  pokeInt(i, pos, 5);
end;

Procedure THPStream.PokeInt(i: int64; pos: integer; count: integer);
Begin
  While count > 0 Do
  Begin
    Data[pos] := i And $F;
    i := i Div 16;
    inc(pos);
    dec(count);
  End;
End;

Function THPStream.PData: PByte;
Begin
  result := @(data[0]);
End;

Procedure THPStream.PokeInt(i: integer; pos: integer);
Begin
  PokeInt(i, pos, 5);
End;

Function THPStream.Read(Var Buffer; Count: Integer): Longint;
//var
//  i:integer;
Begin
  result := min(count, size - position);
  move(data[position], buffer, result);
  position := position + count;
End;

Function THPStream.readInt: integer;
Var
  b: byte;
Begin
  read(b, 1);
  result := b;
  read(b, 1);
  result := result + b * $10;
  read(b, 1);
  result := result + b * $100;
  read(b, 1);
  result := result + b * $1000;
  read(b, 1);
  result := result + b * $10000;
End;

Function THPStream.readInt(n: integer): int64;
Var
  b: byte;
  j: integer;
Begin
  result := 0;
  For j := 0 To n - 1 Do
  Begin
    read(b, 1);
    result := result + b Shl (j * 4)
  End;
End;

Procedure THPStream.SaveToHPFile(f: TFileName; HPHP:string='HPHP48-X');      // wgg
Var
  i: integer;
  d: string;
Begin
  d := HPHP;                                 // wgg was 'HPHP48-X';
  SetLength(d, (length(data) + 1) Div 2 + 8);
  For i := 0 To length(data) - 1 Do
    If i And $1 = 0 Then
      d[i Div 2 + 9] := char(data[i] And $F)
    Else
      d[i Div 2 + 9] := char(ord(d[i Div 2 + 9]) Or (data[i] * 16));
  With TFileStream.Create(f, fmCreate) Do
  Try
    Write(pchar(d)^, length(d));
  Finally
    free;
  End;
End;

Function THPStream.Seek(Offset: Integer; Origin: Word): Longint;
Begin
  Case Origin Of
    soFromBeginning: FPosition := Offset;
    soFromCurrent: FPosition := FPosition + Offset;
    soFromEnd: FPosition := High(Data) - Offset + 1;
  End;
  If FPosition < Low(Data) Then FPosition := 0;
  If FPosition > High(Data) Then FPosition := High(Data) + 1;
  result := FPosition;
End;

Procedure THPStream.SetSize(NewSize: Integer);
Begin
  SetLength(Data, NewSize);
End;

Function THPStream.Write(Const Buffer; Count: Integer): Longint;
//var
//  i: integer;
//  p: Pchar;
Begin
//  p:= pchar(buffer);
  If length(data) < count + FPosition Then
    SetLength(data, FPosition + count);
//  for i:=0 to count-1 do
//    data[FPosition+i]:= ord(p[i]) and $f;
  move(buffer, data[FPosition], count);
  inc(FPosition, Count);
  result := count;                                      
End;

Procedure TCustomPanel2.resize;
Begin
  invalidate;
  Inherited;
End;

Destructor TCustomPanel2.Destroy;
Begin
  FCanvas.Free;
  Inherited;
End;

Function TCustomPanel2.GetCanvas: TCanvas;
Begin
  Result := FCanvas;
End;

Constructor TCustomPanel2.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csSetCaption, csOpaque, csDoubleClicks, csReplicatable];
  Width := 185;
  Height := 41;
  Color := clBtnFace;
  FOnPaint := Nil;
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;
End;


Procedure TCustomPanel2.Paint;
Begin
  If Assigned(FOnPaint) Then FOnPaint(Self);
End;

Procedure TCustomPanel2.PaintWindow(DC: HDC);
Begin
  FCanvas.Lock;
  Try
    FCanvas.Handle := DC;
    Try
      Paint;
    Finally
      FCanvas.Handle := 0;
    End;
  Finally
    FCanvas.Unlock;
  End;
End;

Function GetFileVersion: String;
Var
  H, s, Vl: DWORD;
  D, V: pointer;
Begin
  s := GetFileVersionInfoSize(Pchar(Application.ExeName), H);
  GetMem(D, s);
  Result := '-.-.-.-';
  If GetFileVersionInfo(Pchar(Application.ExeName), H, s, D) Then
    If VerQueryValue(D, '\', V, Vl) Then
      With TVSFixedFileInfo(V^) Do
        GetFileVersion := Format('Version %d.%d Build %d', [dwFileVersionMS Shr 16,
          $0000FFFF And dwFileVersionMS, $0000FFFF And dwFileVersionLS]);      // wgg was Version %d SubVer. %d Build %d
  FreeMem(D);
End;

Function SendThroughRS232(Com: String; Data: String): boolean;
Var
  dcb: _DCB;
  w: DWord;
  i: THandle;
Begin
  result := False;
  i := CreateFile(pchar(com), GENERIC_READ Or GENERIC_WRITE, 0, Nil, OPEN_EXISTING, 0, 0);
  If i = INVALID_HANDLE_VALUE Then exit;
  Try
    If Not GetCommState(i, dcb) Then exit;
    dcb.BaudRate := 9600;
    dcb.Flags := 1;
    dcb.ByteSize := 8;
    dcb.Parity := 0;
//    dcb.StopBits:= 1;
    If Not SetCommState(i, dcb) Then exit;
    While length(data) > 0 Do
    Begin
      If Not WriteFile(i, pchar(Data)^, min(Length(Data), 100), w, Nil) Then exit;
      If integer(w) <> min(Length(Data), 100) Then Exit;
      data := copy(data, 101, MaxInt);
      If Length(Data) <> 0 Then sleep(100);
    End;
    result := True;
  Finally
    CloseHandle(i);
  End;
End;

{Function THPStream.WriteGrob(p: TDib8; gray: boolean = false): integer;
Var
  x, y: integer;
  b: Array Of byte;
Begin
  writeInt($2B1E, 5);
  If gray Then
    result := WriteInt(((p.Width + 7) Div 8) * p.height * 4 + 15)
  Else
    result := WriteInt(((p.width + 7) Div 8) * p.height * 2 + 15);
  WriteInt(p.height);
  WriteInt(p.Width);
  SetLength(b, (p.width + 7) Div 8 * 2);
  For y := 0 To p.height - 1 Do
  Begin
    For x := 0 To high(b) Do b[x] := $F;
    For x := 0 To p.width - 1 Do
      If p.Pixels[x, p.height - y] In [12, 13] Then
        b[x Div 4] := b[x Div 4] - (1 Shl (x And $3));
    write(b[0], length(b));
  End;
  If gray Then
    For y := 0 To p.height - 1 Do
    Begin
      For x := 0 To high(b) Do b[x] := $F;
      For x := 0 To p.width - 1 Do
        If p.Pixels[x, p.height - y] In [11, 13] Then
          b[x Div 4] := b[x Div 4] - (1 Shl (x And $3));
      write(b[0], length(b));
    End;
End;  }

Function THPStream.WriteInt(i: int64; count: integer): longint;
Var
  b: byte;
  j: integer;
Begin
  result := 0;
  For j := 0 To count - 1 Do
  Begin
    b := i And $F;
    result := write(b, 1);
    i := i Shr 4;
  End;
End;

Function THPStream.WriteInt(i: integer): longint;
Begin
  result := WriteInt(i, 5);
End;


Function THPStream.WriteName(s: String): integer;
Begin
  If length(s) > 128 Then Raise exception.Create('Error, name too long');
  WriteInt(length(s), 2);
  WriteString(s);
  result := WriteInt(length(s), 2); 
End;

Function THPStream.WriteString(s: String): integer;
Var
  i: integer;
  s2: Array Of byte;
Begin
  SetLength(s2, length(s) * 2);
  For i := 0 To length(s) - 1 Do
  Begin
    s2[i * 2] := ord(s[i + 1]) And $F;
    s2[i * 2 + 1] := (ord(s[i + 1]) Div 16) And $F;
  End;
  result := Write(pchar(s2)^, length(s2));
End;

Function THPStream.WriteStringObj(s: String): integer;
Begin
  WriteInt($02A2C);                     // String
  WriteInt(length(s) * 2 + 5);          // Size
  result := WriteString(s);
End;

procedure THPStream.LoadFromByteString(const s: string);
var
  i: integer;
begin
  FPosition:= 0;
  SetLength(Data, 0);
  for i:=1 to length(s) do
    WriteInt(ord(s[i]), 2);
end;

function THPStream.GetByteString: string;
Var
  d: String;
  i: integer;
Begin
  d := '';
  SetLength(d, (length(data) + 1) Div 2);
  For i := 0 To length(data) - 1 Do
    If i And $1 = 0 Then
      d[i Div 2 + 1] := char(data[i] And $F)
    Else
      d[i Div 2 + 1] := char(ord(d[i Div 2 + 1]) Or (data[i] * 16));
  result:= d;
End;

{ THexaView }

constructor THexaView.Create(AOwner: TComponent);
begin
  Inherited Create(AOwner);
  FNibbles:= False;
  FReadOnly:= true;
  FCurrentTop:= 0;
  FNbBytePerLine:= 8;
  FMaxAdr:= $100000-1;
  FMinAdr:= 0;
  FCurrentEdit:= 0;
  FAdrWidth:= 5;
  Font.Name:= 'Courier New';
  FPaint:= False;
  ed:= TEdit.Create(self);
  ed.SetBounds(0,0,0,0);
  ed.Parent:= self;
  ed.OnKeyPress:= edKeyPress;
  ed.OnKeyDown:= edKeyDown;
end;

destructor THexaView.destroy;
begin
  ed.free;
  inherited destroy;
end;

procedure THexaView.EdKeyDown(Sender: TObject; var key: word;
  Shift: TShiftState);
var
  shft: integer;
begin
  if Nibbles then
    shft:= FNbBytePerLine*2
  else
    shft:= FNbBytePerLine;
  case key of
    VK_UP:
      Begin
        if CurrentEdit-Shft<CurrentTop then
          CurrentTop:= CurrentTop-shft;
        CurrentEdit:= CurrentEdit-shft;
      end;
    VK_DOWN:
      Begin
        if CurrentEdit+Shft>=CurrentTop+shft*NbLines then
          CurrentTop:= CurrentTop+shft;
        CurrentEdit:= CurrentEdit+shft;
      end;
    VK_LEFT:   CurrentEdit:= CurrentEdit-1;
    VK_RIGHT:  CurrentEdit:= CurrentEdit+1;
  end;
end;

procedure THexaView.edKeyPress(Sender: TObject; var Key: char);
begin
  inherited KeyPress(Key);
  if (not ReadOnly) and Assigned(OnWrite) and ((HexaEdit and (UpperCase(Key)[1] in ['0'..'9','A'..'F'])) or (Not HexaEdit)) then
    If HexaEdit then
    Begin
      OnWrite(Self, CurrentEdit, pos(upperCase(Key), '0123456789ABCDEF')-1, HexaEdit);
      CurrentEdit:= CurrentEdit+1;
    end else begin
      OnWrite(Self, CurrentEdit, ord(Key), HexaEdit);
      if Nibbles then
        CurrentEdit:= CurrentEdit+2
      else
        CurrentEdit:= CurrentEdit+1;
    end;
  invalidate;
end;

procedure THexaView.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if (FPaint) and (mbLeft = Button) then
  Begin
    x:= (x-2) div FWidth - AdrWidth - 1;
    y:= (y-1) div FHeight;
    if (y>=NbLines) or (y<0) or (x>NbBytePerLine*3+AdrWidth+1) then exit;
    ed.SetFocus;
    if x>=0 then
      if nibbles then
        if x<NbBytePerLine*2 then
        Begin
          FHexaEdit:= true;
          CurrentEdit:= CurrentTop+y*NbBytePerLine*2+x;
        end else Begin
          if x>NbBytePerLine*2 then
          begin
            FHexaEdit:= false;
            CurrentEdit:= CurrentTop+y*NbBytePerLine*2+min(NbBytePerLine-1, x-NbBytePerLine*2-1)*2;
          end
        end else
        if x<NbBytePerLine*2 then
        Begin
          FHexaEdit:= true;
          CurrentEdit:= CurrentTop+y*NbBytePerLine+(x div 2);
        end else
          if x>NbBytePerLine*2 then
          Begin
            FHexaEdit:= false;
            CurrentEdit:= CurrentTop+y*NbBytePerLine+min(NbBytePerLine-1, x-NbBytePerLine*2-1);
          end;
  end;
end;

procedure THexaView.Paint;
var
  AdrMul: integer;
  b: array of byte;
  s: string;
  bt: TBitmap;
  i, j, x, y: integer;
  NoRead: boolean;
begin
  bt:= TBitmap.Create;
  try
    NoRead:= CsDesigning in ComponentState;
    bt.width:= width;
    bt.Height:= Height;
    bt.Canvas.brush.color:= color;
    bt.Canvas.Rectangle(0, 0, width, height);
    bt.Canvas.font:= font;
    FHeight:= bt.canvas.TextHeight('W');
    FWidth:= bt.canvas.TextWidth('W');
    NbLines:= (Height-2) div FHeight;
    if NoRead or assigned(OnRead) then
    Begin
      SetLength(b, NbBytePerLine);
      if Nibbles then
        AdrMul:= 1
      else
        AdrMul:= 2;
      for i:= 0 to NbLines-1 do
      Begin
        for j:= 0 to NbBytePerLine-1 do
          if NoRead then
            b[j]:= CurrentTop+i*NbBytePerLine*(3-AdrMul)+j*(3-AdrMul)
          else
            if CurrentTop+i*NbBytePerLine*(3-AdrMul)+j*(3-AdrMul)+1>MaxAdr then
              b[j]:= 0
            else
              b[j]:= OnRead(self, CurrentTop+i*NbBytePerLine*(3-AdrMul)+j*(3-AdrMul));
        s:= IntToHex2(CurrentTop+i*NbBytePerLine*(3-AdrMul), AdrWidth)+' ';
        for j:=0 to NbBytePerLine-1 do
          if Nibbles then
            s:= s+IntToHex2((b[j]*16) and $F0+(b[j] div 16), 2)
          else
            s:= s+IntToHex2(b[j],2);
        s:= s+' ';
        for j:=0 to NbBytePerLine-1 do
          if b[j]<32 then
            s:= s+' '
          else
            s:= s+chr(b[j]);
        bt.Canvas.TextOut(2, i*FHeight+1, s);
      end;

      if (not ReadOnly) and (CurrentEdit>=CurrentTop) and (CurrentEdit<CurrentTop+NbLines*NbBytePerLine*(3-AdrMul)) then
      begin
        i:= (CurrentEdit-CurrentTop) div (NbBytePerLine*(3-AdrMul));
        j:= (CurrentEdit-CurrentTop) - (i*NbBytePerLine*(3-AdrMul));
        y:= i*FHeight+1;
        if HexaEdit then
          X:= (j+6)*FWidth+1
        else
          X:= ((j div 2)+6+17)*FWidth+1;
        bt.canvas.CopyMode:= cmDstInvert;
        bt.Canvas.StretchDraw(rect(x+1, y+1, x+FWidth+1, y+FHeight), bt);
      end;

    end;

    bt.Canvas.Brush.Style:= bsClear;
    bt.Canvas.Rectangle(0, 0, width, height);
    canvas.draw(0,0,bt);
  finally
    bt.free;
  end;
  FPaint:= true;
end;

procedure THexaView.SetAdrWidth(const Value: integer);
begin
  FAdrWidth := Value;
  Invalidate;
end;

procedure THexaView.SetCurrentEdit(const Value: integer);
var
  shft: integer;
begin
  if Nibbles then
    shft:= FNbBytePerLine*2
  else
    shft:= FNbBytePerLine;

  FCurrentEdit := Value;
  If CurrentEdit<MinAdr then FCurrentEdit:= MinAdr;
  If CurrentEdit>MaxAdr then FCurrentEdit:= MaxAdr;
  if CurrentEdit<CurrentTop then
    FCurrentTop:= CurrentEdit;
  if FCurrentTop>= MaxAdr - shft*NbLines then            // wgg limit max top
    FCurrentTop:=  MaxAdr - shft*NbLines;
  if Nibbles then
  Begin
    if FPaint and (CurrentEdit-NbLines*NbBytePerLine*2>=CurrentTop) then
      CurrentTop:= CurrentEdit-NbLines*NbBytePerLine*2+1;
  end else begin
    if FPaint and (CurrentEdit-NbLines*NbBytePerLine>=CurrentTop) then
      CurrentTop:= CurrentEdit-NbLines*NbBytePerLine+1;
  end;
  Invalidate;
end;

procedure THexaView.SetCurrentTop(const Value: integer);
begin
//  if Value>MaxAdr+ then Value:= MaxValue;
  if Value<MinAdr then
    FCurrentTop:= MinAdr
  else
    FCurrentTop := Value;
  Invalidate;
end;

procedure THexaView.SetHexaEdit(const Value: boolean);
begin
  FHexaEdit := Value;
  Invalidate;
end;

procedure THexaView.SetMaxAdr(const Value: integer);
begin
  FMaxAdr := Value;
  Invalidate;
end;

procedure THexaView.SetMinAdr(const Value: integer);
begin
  FMinAdr := Value;
  Invalidate;
end;

procedure THexaView.SetNbBytePerLine(const Value: integer);
begin
  FNbBytePerLine := Value;
  Invalidate;
end;

procedure THexaView.SetNibbles(const Value: boolean);
begin
  FNibbles := Value;
  Invalidate;
end;

procedure THexaView.SetOnRead(const Value: TOnHexaRead);
begin
  FOnRead := Value;
  Invalidate;
end;

procedure THexaView.SetOnWrite(const Value: TOnHexaWrite);
begin
  FOnWrite := Value;
end;

procedure THexaView.SetReadOnly(const Value: boolean);
begin
  FReadOnly := Value;
  Invalidate;
end;

procedure TCustomPanel2.WMPaint(var Message: TWMPaint);
begin
//  Include(FControlState, csCustomPaint);
  inherited;
//  Exclude(FControlState, csCustomPaint);
end;

{ TVersionLabel }

procedure TVersionLabel.SetCaption(const Value: string);
begin
  FCaption := Value;
  Inherited Caption:=Value+GetFileVersion;
end;

{ TCreateProcess }

type
  TWaitThread = class (TThread)
  private
    cp : TCreateProcess;
  protected
    Procedure Execute; override;
  public
    Constructor create(cp: TCreateProcess);
  end;

Constructor TWaitThread.create(cp: TCreateProcess);
Begin
  inherited create(true);
  self.cp:= cp;
  FreeOnTerminate:= true;
  resume;
end;

Procedure TWaitThread.Execute;
Begin
  cp.WaitForCompletion;
end;

destructor TCreateProcess.destroy;
begin
  try
    TerminateThread(wthread.handle, 0);
  except
  end;
  inherited destroy;
end;

procedure TCreateProcess.SetRun(const Value: boolean);
var
  ls: TSTARTUPINFO;
  lp: TProcessInformation;
  rundir: string;
begin
  if Value <> Run then
    if Value then
    begin
      with ls do
      begin
        cb:= sizeof(ls);
        lpReserved:= nil;
        lpDesktop:= nil;
        lpTitle:= nil;
        dwX:= Screen.DesktopWidth;
        dwY:= Screen.DesktopHeight;
        dwXSize:= 0;
        dwYSize:= 0;
        dwXCountChars:= 0;
        dwYCountChars:= 0;
        dwFillAttribute:= 0;
        dwFlags:= 0;
        if FHideChild then
          dwFlags:= STARTF_USESIZE or STARTF_USEPOSITION;
        wShowWindow:= 0;
        cbReserved2:= 0;
        lpReserved2:= nil;
        if RedirectIO then
        begin
          CreatePipe(StdIn2, StdIn1, nil, 0);
          CreatePipe(StdOut1, StdOut2, nil, 0);
          CreatePipe(StdErr1, StdErr2, nil, 0);
          DuplicateHandle(GetCurrentProcess, StdIn2, GetCurrentProcess( ), nil, 0, false, DUPLICATE_SAME_ACCESS);
          DuplicateHandle(GetCurrentProcess, StdOut2, GetCurrentProcess( ), nil, 0, false, DUPLICATE_SAME_ACCESS);
          DuplicateHandle(GetCurrentProcess, StdErr2, GetCurrentProcess( ), nil, 0, false, DUPLICATE_SAME_ACCESS);
          hStdInput:= StdIn2;
          hStdOutput:= StdOut2;
          hStdError:= StdErr2;
          dwFlags:= dwFlags or STARTF_USESTDHANDLES;
        end;
      end;
      if RunningDir = '' then
        RunDir:= ExtractFilePath(FileName)
      else
        RunDir:= RunningDir;
      FileName:= Removeslashslash(FileName);
      CommandLine:= Removeslashslash(CommandLine);
      RunDir:= Removeslashslash(RunDir);
      FRun:= createprocess(pchar(FileName), pchar(CommandLine), nil, nil, true, 0, nil, pchar(RunDir), ls, lp);
      if FRun then
      Begin
        h:= lp.hProcess;
        if FWait then
          WaitForCompletion
        else
          wthread:= TWaitThread.create(self);
      end else
        Application.MessageBox(pchar(inttostr(GetLastError)), '', mb_ok);
    end else begin
      frun:= false;
      if h<>0 then
        if not TerminateProcess(h, 0) then
          raise exception.create('Error while stopping the process');
    end;
end;

Procedure TCreateProcess.WaitForCompletion;
  procedure DoWrite(h: THandle; cb: TWriteProc);
  var
    s: string;
    i: integer;
    j: DWORD;
  begin
    if PeekNamedPipe(H, nil, 0, nil, @i, nil) then
      if i<>0 then
      Begin
        SetLength(s, i);
        ReadFile(H, s, i, j, nil);
        if Assigned(cb) and (length(s)<>0) then
          cb(self, s);
      end;
  end;
var
  h: array [0..3] of THandle;
  i: DWORD;
  quit: boolean;
  s: string;
Begin
  try
    if not RedirectIO then
    Begin
      if WAIT_FAILED=WaitforSingleObject(self.h, INFINITE) then
        raise Exception.create('Error while waiting for completion');
    end else begin
      h[0]:= self.h;
      h[1]:= StdIn1;
      h[2]:= StdOut1;
      h[3]:= StdErr1;
      quit:= false;
      repeat
        i:= WaitForMultipleObjects(4, @(h[0]), false, 1000);
        {$WARNINGS OFF}
        if (i>=WAIT_OBJECT_0) and (i<WAIT_OBJECT_0+4) then
        {$WARNINGS ON}
          case i-WAIT_OBJECT_0 of
            0: quit:= true;
            1: if assigned(OnRead) then
               begin
                 s:= OnRead(self);
                 if length(s)<>0 then
                   WriteFile(h[1], s, length(s), i, nil);
               end;
            2: DoWrite(h[2], OnWrite);
            3: DoWrite(h[2], OnError);
          end;
        if not quit then
        begin
          GetExitCodeProcess(self.h, FExitCode);
          quit:= FExitCode <> STILL_ACTIVE;
        end;
      until quit;
      DoWrite(h[2], OnWrite);
      DoWrite(h[2], OnError);
    end;

    if not GetExitCodeProcess(self.h, FExitCode) then
      raise Exception.create('Error while getting the exit code');

  finally
    FRun:= false;
    CloseHandle(StdIn1);
    CloseHandle(StdIn2);
    CloseHandle(StdOut1);
    CloseHandle(StdOut2);
    CloseHandle(StdErr1);
    CloseHandle(StdErr2);
    CloseHandle(self.h);
  end;
end;

procedure TCreateProcess.Write(const Value: String);
var
  i: DWORD;
begin
  if Run and RedirectIO then
    WriteFile(StdIn1, pchar(Value)^, length(value), i, nil);
end;

{ THP38Dir }

constructor THP38Dir.create;
begin
  inherited create;
  List:= TList.Create;
end;

destructor THP38Dir.destroy;
begin
  List.Free;
  inherited;
end;

procedure THP38Dir.EmptyList;
begin
  while List.count<>0 do
  begin
    TObject(List[0]).free;
    List.Delete(0)
  end;
end;

function THP38Dir.GetCount: integer;
begin
  result:= List.count;
end;

function THP38Dir.GetItem(i: integer): T38Object;
begin
  result:= TObject(List[i]) as T38Object;
end;

procedure THP38Dir.LoadFromFile(f: TFileName);
var
  s: string;
  i: integer;
  o: T38Object;
begin
  EmptyList;
  s:= FileToString(f);
  s:= StringReplace(s, #13#10, ' ');
  s:= StringReplace(s, #10, ' ');
  s:= StringReplace(s, #13, ' ');
  // skip the HP38 header
  s:= copy(s, 9, MaxInt);
  s:= copy(s, SkipWhite(s, 1), MaxInt);
  while length(s)<>0 do
  begin
    o:= T38Object.Create;
    List.add(o);
    o.typ:= s[1];
    s:= copy(s, SkipWhite(s, FindWhite(s, 1)), maxint);  // skip the type
    i:= StrToInt(GetToken(s, 1));                        // Get the PC name size
    s:= copy(s, SkipWhite(s, FindWhite(s, 1)), maxint);  // Skip it
    o.PCName:= copy(s, 1, i);                            // save it
    s:= copy(s, i+1, MaxInt);                            // skip the PC file name
    i:= StrToInt(GetToken(s, 1));                        // Get the HP file name size
    s:= copy(s, SkipWhite(s, FindWhite(s, 1)), maxint);  // skip the HP file name size
    o.HPName:= copy(s, 1, i);                            // save it
    s:= copy(s, i+1, MaxInt);                            // skip it
    s:= copy(s, SkipWhite(s, 1), maxint);
  end;
end;

procedure THP38Dir.SaveToFile(f: TFileName; h: String);
var
  s: string;
  i: integer;
begin
  s:= 'HP'+h+'AscA ';
  for i:=0 to count-1 do
    s:= s+Items[i].typ+' '+inttostr(Length(Items[i].PCName))+' '+Items[i].PCName+inttostr(Length(Items[i].HPName))+' '+Items[i].HPName;
  StringToFile(s, f);
end;

{ TEditLabel }

procedure TEditLabel.checkValue;
begin
  if DecOnly or HexOnly then
    if (Value>FMax) or (Value<FMin) then
      value:= min(max(value, fMax), fmin);
end;

constructor TEditLabel.create(AOwner: TComponent);
begin
  inherited;
  ed:= TEdit.Create(self);
  ed.SetBounds(0,0,0,0);
  ed.Parent:= Parent;
  ed.OnKeyPress:= edKeyPress;
  ed.OnKeyDown:= edKeyDown;
  ed.OnExit:= EdExit;
end;

destructor TEditLabel.destroy;
begin
  ed.free;
  inherited;
end;

procedure TEditLabel.DoDrawText(var Rect: TRect; Flags: Integer);
var
  Text: string;
  i, w, m: integer;
  r: TRect;
  a: array [0..2] of TPoint;
begin
  Text := caption+editzone;
  if (Flags and DT_CALCRECT <> 0) and (Text = '') then Text := Text + ' ';
  Flags := Flags or DT_NOPREFIX;
  Flags := DrawTextBiDiModeFlags(Flags);
  Canvas.Font := Font;

  if (Flags and DT_CALCRECT <> 0) then
  begin
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    if SpinEdit then
      Rect.Right:= Rect.Right+10;
    exit;
  end;

  If SpinEdit then
    with canvas do
    begin
      r:= Rect;
      r.Left:= r.Right-9;
      m:= (r.Top+r.Bottom-1) div 2;
      MoveTo(r.Left, m);
      LineTo(r.Right, m);
      MoveTo(r.Left, r.Top);
      LineTo(r.Right-1, r.Top);
      LineTo(r.Right-1, r.Bottom-1);
      LineTo(r.Left, r.Bottom-1);
      LineTo(r.Left, r.Top);
      Brush.Color:= clBlack;
      a[0].x:= r.Left+2;
      a[0].y:= m-2;
      a[1].x:= (r.Left+r.Right-1) div 2;
      a[1].y:= r.top+2;
      a[2].x:= r.Right-3;
      a[2].y:= m-2;
      Polygon(a);
      a[0].x:= r.Left+2;
      a[0].y:= m+2;
      a[1].x:= (r.Left+r.Right-1) div 2;
      a[1].y:= r.Bottom-3;
      a[2].x:= r.Right-3;
      a[2].y:= m+2;
      Polygon(a);
      Brush.Color:= Color;
      SpinButs[0]:= r; SpinButs[0].Bottom:= m-1;
      SpinButs[1]:= r; SpinButs[1].Top:= m;
    end;

  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end else begin
      DrawText(Canvas.Handle, PChar(caption), Length(caption), Rect, Flags);
      rect.left:= rect.left+canvas.textwidth(caption);
      canvas.font.color:= EditColor;
      if Length(poses)<>length(EditZone) then SetLength(poses, Length(EditZone));
      for i:=1 to length(EditZone) do
      begin
        if i=EditPos then Canvas.Font.Style:= Canvas.Font.Style + [fsBold];
        Text:= EditZone[i];
        drawText(canvas.Handle, Pchar(Text), Length(Text), Rect, Flags);
        w:= canvas.textwidth(Text);
        poses[i-1]:= rect;
        poses[i-1].Right:= rect.left+w-1;
        rect.left:= rect.left+w;
        if i=EditPos then Canvas.Font.Style:= Canvas.Font.Style - [fsBold];
      end;
  end;
end;

procedure TEditLabel.EdExit(Sender: TObject);
begin
  EditPos:= 0;
end;

procedure TEditLabel.EdKeyDown(Sender: TObject; var key: word; Shift: TShiftState);
begin
  if Assigned(OnKeyDown) then OnKeyDown(self, Key, shift);
  case key of
    VK_LEFT:   EditPos:= max(1, EditPos-1);
    VK_RIGHT:  EditPos:= min(Length(EditZone), EditPos+1);
    VK_UP:     if SpinEdit then Value:=Value+1;
    VK_DOWN:   if SpinEdit then Value:=Value-1;
  end;
end;

procedure TEditLabel.EdKeyPress(Sender: TObject; var Key: char);
var
  t: string;
begin
  if Assigned(FKeyPress) then FKeyPress(self, Key);
  if HexOnly then Key:= Uppercase(Key)[1];
  if (EditPos in [1..Length(EditZone)]) and
     ((not HexOnly) or (Key in ['0'..'9','A'..'F'])) and
     ((not DecOnly) or (Key in ['0'..'9'])) then
  begin
    t:= EditZone;
    t[EditPos]:= Key;
    EditPos:= min(Length(EditZone), EditPos+1);
    EditZone:= t;
  end;
  if Key in [#7, #8] then EditPos:= max(1, EditPos-1);
  if Assigned(FAfterKeyPress) then FAfterKeyPress(self, Key);
end;

procedure TEditLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  i: integer;
begin
  inherited;
  If Button = mbLeft then
  begin
    for i:=low(poses) to high(poses) do
      if (X in [poses[i].Left..Poses[i].Right]) and (Y in [poses[i].top..poses[i].Bottom]) then
      begin
        EditPos:= i+1;
        ed.Parent:= Parent;
        if ed.parent <> nil then
          ed.SetFocus;
        exit;
      end;
    if (X in [SpinButs[0].Left..SpinButs[0].Right]) and (Y in [SpinButs[0].top..SpinButs[0].Bottom]) then
    begin
      Value:= Value+1;
      ed.Parent:= Parent;
      if ed.parent <> nil then
        ed.SetFocus;
    end;
    if (X in [SpinButs[1].Left..SpinButs[1].Right]) and (Y in [SpinButs[1].top..SpinButs[1].Bottom]) then
    begin
      Value:= Value-1;
      ed.Parent:= Parent;
      if ed.parent <> nil then
        ed.SetFocus;
    end;
  end;
end;

procedure TEditLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect, CalcRect: TRect;
  DrawStyle: Longint;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    { DoDrawText takes care of BiDi alignments }
    DrawStyle := DT_EXPANDTABS or Alignments[Alignment];
    { Calculate vertical layout }
    if Layout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if Layout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    DoDrawText(Rect, DrawStyle);
  end;
end;

function TEditLabel.ReadValue: int64;
var
  i: integer;
begin
  try
    if HexOnly then
    begin
      val('$'+EditZone, result, i);
      if i<>0 then
        result:= -1;
    end else
      result:= strtoint(EditZone);
  except
    result:= -1;
  end;
end;

procedure TEditLabel.SetEditColor(const Value: TColor);
begin
  FEditColor := Value;
  Invalidate;
end;

procedure TEditLabel.SetEditPos(const Value: integer);
begin
  FEditPos := Value;
  Invalidate;
end;

procedure TEditLabel.SetEditZone(const Value: string);
begin
  FEditZone := Value;
  Invalidate;
  AdjustBounds;
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TEditLabel.Setmax(const Value: integer);
begin
  Fmax := Value;
  checkValue;
end;

procedure TEditLabel.Setmin(const Value: integer);
begin
  Fmin := Value;
  checkValue;
end;

procedure TEditLabel.SetSpinEdit(const Value: boolean);
begin
  FSpinEdit := Value;
  Invalidate;
  AdjustBounds;
end;

procedure TEditLabel.WriteValue(const Value: int64);
var
  s: string;
begin
  if HexOnly then
    s:= inttohex(value, 16)
  else
    s:= inttostr(value);
  if length(s)>length(EditZone) then
    s:= copy(s, length(s)-length(EditZone)+1, length(EditZone));
  while length(s)<length(EditZone) do s:= '0'+s;
  EditZone:= s;
end;

initialization
  CrcInitTable;

End.

