unit windowsPosSave;
// wgg Changed to get registry main key from Application.tag
//
interface

uses
  Classes, Controls, Forms, registry;

type
  TWindowPosSave = class(TComponent)
  protected
    FComp: TStrings;
    FMKey, FSKey: string;
    procedure SetComp(st: TStrings);
  public
    constructor Create(aowner: tcomponent); override;
    destructor Destroy; override;
    Procedure ReadPos(ForceInvisible: boolean = false; LoadControl: boolean = true; LoadMain: boolean = true);
    procedure SavePos(ForceInvisible: boolean = false);                     // wgg added ForceInvisible
  published
    property ImportantComponents: TStrings read FComp write SetComp;
    property MainKey: string read FMKey write FMKey;
    property SubKey: string read FSKey write FSKey;
  end;

procedure Register;

implementation

procedure TWindowPosSave.ReadPos(ForceInvisible: boolean = false; LoadControl: boolean = true; LoadMain: boolean = true);
var
  r: TRegIniFile;
  sec, RegKey: String;
  width, height: Integer;

  Procedure LoadControlPos(r: TRegIniFile; c:TControl; ssec: String);
  begin
    if (ssec = '') and                                                            // wgg if a resizeable parent form
         ((owner as TForm).BorderStyle <> bsSizeable) and ((owner as TForm).BorderStyle <> bsSizeToolWin) then
    begin                                                                       // wgg dont restore size of fixed windows
      width:= c.Width;                                                          // else problem when we change code!
      height:= c.Height;
    end else
    begin
      width:= r.ReadInteger(sec,ssec+'Width',c.width);
      height:= r.ReadInteger(sec,ssec+'Height',c.height);
    end;
    c.SetBounds(r.ReadInteger(sec,ssec+'Left',c.Left), r.ReadInteger(sec,ssec+'Top',c.top), width, height);
    try
      c.Visible:= r.ReadBool(sec, ssec+'Visible', c.Visible) And (Not ForceInvisible);
    except
    end;
  end;

  Procedure LoadControls(c: TWinControl);
  var
    j: integer;
    ct: TControl;
  begin
    for j := 0 to c.ControlCount-1 do    // Iterate
    begin
      ct:= c.Controls[j];
      if FComp.IndexOf(ct.Name)<>-1 then
        LoadControlPos(r,ct,ct.Name);
      if ct is TWinControl then
        LoadControls(ct as TWinControl);
    end;
  end;

begin
  RegKey:= String(Pointer(Application.Tag)^);
  if owner is TForm then
  begin
    sec:= SubKey;                                                // wgg was (owner as TForm).name;
    r:= TRegIniFile.Create(RegKey);                         // wgg override was RegistryKey
    try
      if LoadMain then
      begin
        LoadControlPos(r,owner as TControl,'');
        (owner as TForm).font.Name:= r.ReadString(sec,'Font',(owner as TForm).font.Name);
        (owner as TForm).Font.Height:= r.ReadInteger(sec,'FontSize',(owner as TForm).font.Height);
      end;
      if LoadControl then
        LoadControls(owner as TWinControl);
    finally
      r.free;
    end;
  end;
end;

//  wgg if ForceInvisible is True, then registry Visible will be forced false
procedure TWindowPosSave.SavePos(ForceInvisible: boolean = false);
var
  r: TRegIniFile;
  sec, RegKey: String;

  Procedure SaveControlPos(r: TRegIniFile; c:TControl; ssec: String);
  begin
    r.WriteBool(sec, ssec+'Visible', c.Visible And Not ForceInvisible);
    r.WriteInteger(sec,ssec+'Top',c.top);
    r.WriteInteger(sec,ssec+'Left',c.Left);
    if ssec = '' then
    begin
      if ((owner as TForm).BorderStyle = bsSizeable) or ((owner as TForm).BorderStyle = bsSizeToolWin) then
      begin                                                                       // wgg dont save size of fixed windows
        r.WriteInteger(sec,ssec+'Width',c.Width);                                 // else problem when we change code!
        r.WriteInteger(sec,ssec+'Height',c.Height);
      end;
    end
    else
    begin
      r.WriteInteger(sec,ssec+'Width',c.Width);
      r.WriteInteger(sec,ssec+'Height',c.Height);
    end;
  end;

  Procedure SaveControls(c: TWinControl);
  var
    j: integer;
    ct: TControl;
  begin
    for j := 0 to c.ControlCount-1 do    // Iterate
    begin
      ct:= c.Controls[j];
      if FComp.IndexOf(ct.Name)<>-1 then
        SaveControlPos(r,ct,ct.Name);
      if ct is TWinControl then
        SaveControls(ct as TWinControl);
    end;
  end;

begin
  if owner is TForm then
  begin
    RegKey:= String(Pointer(Application.Tag)^);
    sec:= SubKey;                                     // wgg was (owner as TForm).name
    r:= TRegIniFile.Create(RegKey);                   // wgg override was RegistryKey
    try
      SaveControlPos(r,owner as TControl,'');
      ForceInvisible:= false;                         // let control state do rest
      r.WriteString(sec,'Font',(owner as TForm).font.Name);
      r.WriteInteger(sec,'FontSize',(owner as TForm).font.Height);
      SaveControls(owner as TWinControl);
    finally
      r.free;
    end;
  end;
end;

procedure TWindowPosSave.SetComp(st: TStrings);
begin
  FComp.Assign(st);
end;

constructor TWindowPosSave.create(aowner: tcomponent);
begin
  inherited create(aowner);
  Fcomp:= TStringList.Create;
end;

destructor TWindowPosSave.Destroy;
begin
  Fcomp.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('CDBShort', [TWindowPosSave]);                              //wgg was exemples
end;

end.
