unit DIB;

interface
uses windows, sysUtils, math;

type

  TPoints3d = record
    x,y,z,w:single;
    nx,ny,nz:single;
    xr,yr,zr:single;
    nxr,nyr,nzr:single;
    tx,ty:single;
  end;
  PPoints3d = ^TPoints3d;

  function Point3d2f(x,y:single):TPoints3d;
  function Point3d3f(x,y,z:single):TPoints3d;
  function Point3d6f(x,y,z,nx,ny,nz:single):TPoints3d;
  function Point3d5f(x,y,z,tx,ty:single):TPoints3d;
  function Point3d8f(x,y,z,nx,ny,nz,tx,ty:single):TPoints3d;
  function Point3d3fnz(x,y,z:single):TPoints3d;

type
  TFace3d = record
    p1,p2,p3:PPoints3d;
    nx,ny,nz:single;
    nxr,nyr,nzr:single;
    z:single;
  end;

  TDrawProc = Procedure (const f:TFace3d);

  procedure normaleFace(var f:TFace3d);
  procedure ExecFaceZ(FList: array of TFace3D; drawProc:TDrawProc);
  procedure CalcZ(var v1:array of TFace3d);
  function Face3dcn(p1,p2,p3:PPoints3d):TFace3d;

type
  TDIB8 = class (tobject)
  protected
    BI : PBitmapInfo;
    IHeight, IWidth : integer;
    procedure RecalcSize;
    procedure SetWidth(Width:integer);
    procedure SetHeight(Height:integer);
    Procedure SetPixel(x,y:integer; index:byte);
    function GetPixel(x,y:integer):byte;
    procedure SetPal(i:integer; color:integer);
    function GetPal(i:integer):integer;
  public
    Pxls : pointer;
    constructor create;
    destructor Destroy; override;
    procedure FlipHorizontal;
    procedure Draw(x,y:integer; Source:TDIB8);
    Procedure CopyPalette(Source:TDIB8);
    procedure AssociationPal(dc:hdc);
    procedure Triangle(pt1,pt2,pt3:TPoints3d; color:byte);
    procedure Gourau(pt1,pt2,pt3:TPoints3d; c1,c2,c3:byte);
    procedure Map(pt1,pt2,pt3:TPoints3d; dib:tdib8);
    procedure clear(color:byte);
    Function LoadFromFile(FileName:TFileName):boolean;
    function DrawToDC(DC:Hdc; x,y:integer):integer;
    function mandel(x1,y1,x2,y2:Double):integer;
    function mandel2(x1,y1,x2,y2:double; b:integer):integer;
    property width:integer read IWidth write SetWidth;
    property Height:integer read IHeight write SetHeight;
    property pixels[x,y:integer]:byte read GetPixel write SetPixel;
    property colors[i:integer]:integer read GetPal write SetPal;
    procedure assign(d: TDib8);
  end;

  TTransMat = class
  protected
  public
    matrix : array [0..3,0..3] of single;
    constructor create;
    procedure rotate(x,y,z:single);
    procedure translate(x,y,z:single);
    procedure scale(x,y,z:single);
    procedure identity;
    procedure multiplyPointsPos(var v1:array of TPoints3d);
    procedure multiplyPointsNorm(var v1:array of TPoints3d);
    procedure multiplyFaceNorm(var v1:array of TFace3d);
  end;

implementation

function Point3d2f(x,y:single):TPoints3d;
Begin
  result:=Point3d8f(x,y,0,0,0,0,0,0);
End;

function Point3d3f(x,y,z:single):TPoints3d;
Begin
  result:=Point3d8f(x,y,z,0,0,0,0,0);
End;

function Point3d6f(x,y,z,nx,ny,nz:single):TPoints3d;
Begin
  result:=Point3d8f(x,y,z,nx,ny,nz,0,0);
End;

function Point3d5f(x,y,z,tx,ty:single):TPoints3d;
Begin
  result:=Point3d8f(x,y,z,0,0,0,tx,ty);
End;

function Point3d8f(x,y,z,nx,ny,nz,tx,ty:single):TPoints3d;
Begin
  result.x:=x;
  result.y:=y;
  result.z:=z;
  result.w:=1;
  result.nx:=nx;
  result.ny:=ny;
  Result.nz:=nz;
  result.tx:=tx;
  result.ty:=ty;
  result.xr:=0;
  result.yr:=0;
  result.zr:=0;
  result.nxr:=0;
  result.nyr:=0;
  result.nzr:=0;
End;

function Point3d3fnz(x,y,z:single):TPoints3d;
var s:single;
Begin
  s:=sqrt(x*x+y*y+z*z);
  if s=0 then
    result:=Point3d8f(x,y,z,0,0,0,0,0)
  else
    result:=Point3d8f(x,y,z,x/s,y/s,z/s,0,0);
End;

procedure normaleFace(var f:TFace3d);
var x1,x2,y1,y2,z1,z2,s,x,y,z:single;
Begin
  with f do
  begin
    x2:=p2^.x-p1^.x;
    x1:=p3^.x-p1^.x;
    y2:=p2^.y-p1^.y;
    y1:=p3^.y-p1^.y;
    z2:=p2^.z-p1^.z;
    z1:=p3^.z-p1^.z;
  end;
  x:=y1*z2-y2*z1;
  y:=z1*x2-z2*x1;
  z:=x1*y2-x2*y1;
  s:=sqrt(x*x+y*y+z*z);
  if s=0 then
  Begin
    f.nx:=0;
    f.ny:=0;
    f.nz:=0;
  End
  else
  Begin
    f.nx:=x/s;
    f.ny:=y/s;
    f.nz:=z/s;
  End;
End;

procedure CalcZ(var v1:array of TFace3d);
  procedure calc2(p1:pointer; count:integer);
  asm
    push esi
    push edi
    mov esi,p1
@b1:
    mov edi,[esi]
    fld dword ptr [edi+36]
    mov edi,[esi+4]
    fadd dword ptr [edi+36]
    mov edi,[esi+8]
    fadd dword ptr [edi+36]
    add esi,40
    dec count
    fstp dword ptr [esi+36-40]
    jnz @b1
    pop edi
    pop esi
    finit
  end;
var count : integer;
Begin
  count:=high(v1);
  calc2(@v1[0],count+1);
End;


type
  TPFace3d = ^TFace3d;
  Tarrayofint = array [0..(Maxint div 8) -1] of TPFace3d;
var IntegerTab : ^Tarrayofint;
    IntegerTabUse,IntegerTabSize : integer;
procedure ExecFaceZ(FList: array of TFace3D; drawProc:TDrawProc);
  procedure QuickSort(L, R: Integer);
  var
    I, J: Integer;
    P, T: TPFace3d;
  begin
    repeat
      I := L;
      J := R;
      P := IntegerTab^[(L + R) shr 1];
      repeat
        while IntegerTab^[I]^.z<P^.z do Inc(I);
        while IntegerTab^[J]^.z>P^.z do Dec(J);
        if I <= J then
        begin
          T := IntegerTab^[I];
          IntegerTab^[I] := IntegerTab^[J];
          IntegerTab^[J] := T;
          Inc(I);
          Dec(J);
        end;
      until I > J;
      if L < J then QuickSort(L, J);
      L := I;
    until I >= R;
  end;
var i:integer;
Begin
  if IntegerTabSize<=High(FList) then
    reallocMem(IntegerTab,High(FList)*SizeOf(Integer));
  Begin
    IntegerTabUse:=0;
    for i:=0 to high(FList) do
      if FList[i].nzr>=0 then
      begin
        IntegerTab^[IntegerTabUse]:=@flist[i];
        inc(IntegerTabUse);
      end;
  end;
  if IntegerTabUse<>0 then
  begin
    QuickSort(0,IntegerTabUse-1);
    for i:=0 to IntegerTabUse-1 do
      drawProc(integerTab[i]^);
  end;
End;

function Face3dcn(p1,p2,p3:PPoints3d):TFace3d;
Begin
  result.p1:=p1;
  result.p2:=p2;
  result.p3:=p3;
  normaleFace(result);
  result.nxr:=0;
  result.nyr:=0;
  result.nzr:=0;
  result.z:=0;
End;


procedure TDIB8.assign(d: TDib8);
var
  i: integer;
begin
  width:= d.width;
  height:= d.height;
  for i:=0 to 255 do
    colors[i]:=d.colors[i];
  move(d.pxls^, pxls^, BI^.bmiHeader.biSizeImage);
  move(d.BI^, BI^, sizeof(BI^))
end;

procedure TDIB8.RecalcSize;
Begin
  BI^.bmiHeader.biSizeImage:=width*Height;
  ReallocMem(Pxls,BI^.bmiHeader.biSizeImage);
  if BI^.bmiHeader.biSizeImage and $3 <>0 then
    raise exception.Create('Taille de l''image non multiple de 4');
End;

procedure TDIB8.SetWidth(Width:integer);
Begin
  BI^.bmiHeader.biWidth:=Width;
  IWidth:=Width;
  RecalcSize;
End;

procedure TDIB8.SetHeight(Height:integer);
Begin
  BI^.bmiHeader.biHeight:=Height;
  IHeight:=Height;
  RecalcSize;
End;

Procedure TDIB8.SetPixel(x,y:integer; index:byte);
asm
  push esi
  mov esi,[eax].Pxls
  add esi,x
  mov eax,[eax].Iwidth
  mul y
  mov cl,index
  mov [esi+eax],cl
  pop esi
end;

function TDIB8.GetPixel(x,y:integer):byte;
asm
  push esi
  mov esi,[eax].Pxls
  add esi,x
  mov eax,[eax].Iwidth
  mul y
  mov al,[esi+eax]
  pop esi
end;

procedure TDIB8.SetPal(i:integer; color:integer);
Begin
  if i>255 then
    raise exception.create('Moins de '+inttostr(i)+' couleures dans la palette');
  color:= color and $ffffff;
  bi^.bmiColors[i]:=TRGBQuad(color);
End;

function TDIB8.GetPal(i:integer):integer;
Begin
  if i>255 then
    raise exception.create('Moins de '+inttostr(i)+' couleures dans la palette');
  result:=integer(bi^.bmicolors[i]);
End;

Constructor TDIB8.create;
Begin
  inherited create;
  reallocMem(BI,sizeof(BI^)+255*sizeof(TRGBQuad));
  IWidth:=0;
  IHeight:=0;
  with BI^.bmiHeader do
  Begin
    biSize:=SizeOf(BI^.bmiHeader);
    biWidth:=0;
    biHeight:=0;
    biPlanes:=1;
    biBitCount:=8;
    biCompression:=BI_RGB;
    biSizeImage:=0;
    biXPelsPerMeter:=0;
    biYPelsPerMeter:=0;
    biClrUsed:=256;
    biClrImportant:=256;
  End;
End;

destructor TDIB8.destroy;
Begin
  ReallocMem(Pxls,0);
  ReallocMem(BI,0);
  inherited destroy;
End;

Procedure TDIB8.FlipHorizontal;
Begin
  asm
    push edi
    push ebx
    mov ebx,eax
    mov edi,[ebx].pxls
    mov eax,[ebx].IHeight
    mov ecx,[ebx].IWidth
    mov esi,edi
    mul ecx
    add edi,eax
    mov eax,[ebx].IHeight
    shr eax,1
@b1:
    sub edi,ecx
@b2:
    mov dl,[edi]
    mov dh,[esi]
    mov [edi],dh
    mov [esi],dl
    inc edi
    inc esi
    dec ecx
    jne @b2
    mov ecx,[ebx].IWidth
    sub edi,ecx
    dec eax
    jne @b1
    pop ebx
    pop edi
  end;
End;

procedure TDIB8.Draw(x,y:integer; Source:TDIB8);
var sx,sy, // position du premier point a afficher
    sw,sh:integer; // nombre de points en X et Y a afficher
    wd,ws:integer; // nombre de points a passer en fin de ligne sur la source et la destination
    pxd,pxs:pointer;
    wid,wis:integer; // width destination et source
begin
  if x+source.width<0 then
    exit;
  if x>width then
    exit;
  if y+source.Height<0 then
    exit;
  if y>width then
    exit;
  if x<0 then
    sx:=-x
  else
    sx:=0;
  if y<0 then
    sy:=-y
  else
    sy:=0;
  if x+source.width>width then
    sw:=width-x-sx
  else
    sw:=Source.width-sx;
  if y+Source.Height>Height then
    sh:=Height-y-sy
  else
    sh:=Source.Height-sy;
  pxd:=Pxls;
  pxs:=source.Pxls;
  wid:=width;
  wd:=Wid-sw;
  wis:=source.width;
  ws:=wis-sw;
  asm
    push edi
    mov edi,pxd
    mov eax,y
    mul wid
    add eax,x
    add edi,eax

    mov esi,pxs
    mov eax,sy
    mul wis
    add eax,sx
    add esi,eax

    mov ecx,sh
@B1:
    mov edx,sw
@B2:
    mov al,[esi]
    inc esi
    cmp al,0
    jz @Zero
    mov [edi],al
@Zero:
    inc edi
    dec edx
    jnz @B2
    add esi,ws
    add edi,wd
    dec ecx
    jnz @B1

    pop edi
  end;
end;

Procedure TDIB8.CopyPalette(Source:TDIB8);
var i:integer;
Begin
  for i:=0 to 255 do
    BI^.bmiColors[i]:=source.BI^.bmiColors[i];
End;

procedure TDIB8.AssociationPal(dc:hdc);
var nbsys,i:integer;
    logp:PLogPalette;
Begin
  if (GetDeviceCaps(dc,RASTERCAPS) and RC_PALETTE) <> 0 then
  begin
    logp:=nil;
    reallocMem(logp,sizeof(tlogpalette)+sizeof(TRGBQuad)*255);
    logp^.palVersion:=$300;
    logp^.palNumEntries:=256;
    nbsys:=GetDeviceCaps(dc,NUMRESERVED);
    GetSystemPaletteEntries(dc,0,nbsys shr 1,bi^.bmiColors);
    GetSystemPaletteEntries(dc,255-nbsys shr 1,nbsys shr 1,bi^.bmiColors[255-nbsys shr 1]);
    for i:=0 to 255 do
    begin
      BI^.bmiColors[i].rgbReserved:=0;
      logp^.palPalEntry[i].peRed:=BI^.bmiColors[i].rgbred;
      logp^.palPalEntry[i].peGreen:=BI^.bmiColors[i].rgbGreen;
      logp^.palPalEntry[i].peBlue:=BI^.bmiColors[i].rgbBlue;
      logp^.palPalEntry[i].peFlags:=0;
    end;
    deleteobject(SelectPalette(dc,createPalette(logp^),false));
    RealizePalette(dc);
    reallocMem(logp,0);
  end;
End;


{var
  TriOff, TriNbp, TriIOff, TriINbp, TriH : integer;
  TriC1, TriC2, TriC3, TriIC1, TriIC2, TriIC3 : integer;
  TriMx1, TriMx2, TriMy1, TriMy2, TriIMx1, TriIMx2, TriIMy1, TriIMy2 : integer;
  TriData, TriTex : integer;
}
procedure TDIB8.Triangle(pt1,pt2,pt3:TPoints3d; color:byte);
  procedure BoutTriangle(p1:pointer; inc1,inc2:integer; var v1,v2:integer; color:byte; h:integer);
  Begin
    asm
      push ebx
      push edi
      mov edx,v1
      mov edx,[edx]
      mov ebx,v2
      mov ebx,[ebx]
@B1:
      mov ecx,edx
      mov eax,ebx
      shr ecx,12
      shr eax,12
      mov edi,p1
      add edi,ecx

      std
      sub ecx,eax
      jns @pos
      not ecx
      cld
@pos:
      inc ecx
      mov al,color
      rep StosB
      add edx,inc1
      add ebx,inc2
      dec h
      jnz @B1
      mov ecx,v1
      mov eax,v2
      mov [ecx],edx
      mov [eax],ebx
      cld
      pop edi
      pop ebx
    end;
  End;

  procedure exch(var i,j:integer);
  var tmp:integer;
  Begin
    tmp:=i;
    i:=j;
    j:=tmp;
  End;

var p1,p2,p3,pt:integer;
    inc1,inc2,inc3:integer;
    x1,x2,x3,y1,y2,y3:integer;
Begin
  x1:=round(pt1.xr);
  x2:=round(pt2.xr);
  x3:=round(pt3.xr);
  y1:=round(pt1.yr);
  y2:=round(pt2.yr);
  y3:=round(pt3.yr);
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
  End;
  if y2>y3 then
  Begin
    exch(y2,y3);
    exch(x2,x3);
  End;
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
  End;
  p1:=(y1*Width+x1) shl 12;
  p2:=(y2*Width+x2) shl 12;
  p3:=(y3*Width+x3) shl 12;
  if y1=y2 then
    if y1=y3 then
    begin
      BoutTriangle(pxls,0,0,p3,p1,color,1);
      BoutTriangle(pxls,0,0,p1,p2,color,1)
    end
    else
    Begin
      inc1:=(p3-p1) div (y3-y1);
      inc2:=(p3-p2) div (y3-y2);
      BoutTriangle(pxls,inc1,inc2,p1,p2,color,y3-y1)
    End
  else
  begin
    inc1:=(p3-p1) div (y3-y1);
    inc2:=(p2-p1) div (y2-y1);
    pt:=p1;
    BoutTriangle(Pxls,inc1,inc2,p1,pt,color,y2-y1);
    if y2<>y3 then
    begin
      inc3:=(p3-p2) div (y3-y2);
      BoutTriangle(pxls,inc1,inc3,p1,p2,color,y3-y2);
    end;
  end;
End;

procedure TDIB8.Gourau(pt1,pt2,pt3:TPoints3d; c1,c2,c3:byte);
  procedure BoutTriangle(p1:pointer; inc1,inc2:integer; var v1,v2:integer; var c1,c2:integer; incc1,incc2:integer; h:integer);
  Begin
    asm
      push ebx
      push edi
      push esi
      mov esi,v1
      mov esi,[esi]
      mov ebx,v2
      mov ebx,[ebx]
@B1:
      mov ecx,esi
      mov eax,ebx
      shr ecx,12
      shr eax,12
      mov edi,p1
      add edi,ecx
      std
      sub ecx,eax       //ecx: nombre de pixles de la ligne
      jnc @pzero1
      neg ecx
      cld
@pzero1:
      inc ecx

      mov eax,c2
      mov eax,[eax]     // eax: c1
      mov edx,c1
      mov edx,[edx]     // edx: c2
      push edx
      sub eax,edx       // eax:c1-c2
      cdq
      idiv ecx
      mov edx,eax

      pop eax     // eax: c1

@b2:  rol eax,9
      StosB
      ror eax,9
      add eax,edx
      dec ecx
      jnz @b2

      mov eax,c1
      mov edx,[eax]
      add edx,incc1
      mov [eax],edx
      mov eax,c2
      mov edx,[eax]
      add edx,incc2
      mov [eax],edx

      add esi,inc1
      add ebx,inc2
      dec h
      jnz @B1

      mov ecx,v1
      mov eax,v2
      mov [ecx],esi
      mov [eax],ebx
      cld
      pop esi
      pop edi
      pop ebx
    end;
  End;

  procedure exch(var i,j:integer);
  var tmp:integer;
  Begin
    tmp:=i;
    i:=j;
    j:=tmp;
  End;

  procedure exchc(var i,j:byte);
  var tmp:byte;
  Begin
    tmp:=i;
    i:=j;
    j:=tmp;
  End;

var p1,p2,p3,pt:integer;
    inc1,inc2,inc3:integer;
    cc1,cc2,cc3,cl:integer;
    incc1,incc2,incc3:integer;
    x1,x2,x3,y1,y2,y3:integer;
Begin
  x1:=round(pt1.xr);
  x2:=round(pt2.xr);
  x3:=round(pt3.xr);
  y1:=round(pt1.yr);
  y2:=round(pt2.yr);
  y3:=round(pt3.yr);
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
    exchc(c1,c2);
  End;
  if y2>y3 then
  Begin
    exch(y2,y3);
    exch(x2,x3);
    exchc(c2,c3);
  End;
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
    exchc(c1,c2);
  End;
  p1:=(y1*Width+x1) shl 12;
  p2:=(y2*Width+x2) shl 12;
  p3:=(y3*Width+x3) shl 12;
  cc3:=c3 shl 23;
  cc1:=c1 shl 23;
  cc2:=c2 shl 23;
  if y1=y2 then
    if y1=y3 then
     Begin
       BoutTriangle(pxls,0,0,p3,p1,cc3,cc1,0,0,1);
       BoutTriangle(pxls,0,0,p2,p1,cc2,cc1,0,0,1)
     end
    else
    Begin
      inc1:=(p3-p1) div (y3-y1);
      inc2:=(p3-p2) div (y3-y2);
      incc1:=(cc3-cc1) div (y3-y1);
      incc2:=(cc3-cc2) div (y3-y2);
      BoutTriangle(pxls,inc1,inc2,p1,p2,cc1,cc2,incc1,incc2,y3-y1)
    End
  else
  begin
    inc1:=(p3-p1) div (y3-y1);
    inc2:=(p2-p1) div (y2-y1);
    incc1:=(cc3-cc1) div (y3-y1);
    incc2:=(cc2-cc1) div (y2-y1);
    pt:=p1;
    cl:=cc1;
    BoutTriangle(Pxls,inc1,inc2,p1,pt,cc1,cl,incc1,incc2,y2-y1);
    if y2<>y3 then
    begin
      inc3:=(p3-p2) div (y3-y2);
      incc3:=(cc3-cc2) div (y3-y2);
      BoutTriangle(pxls,inc1,inc3,p1,p2,cc1,cc2,incc1,incc3,y3-y2);
    end;
  end;
End;

procedure TDIB8.Map(pt1,pt2,pt3:TPoints3d; dib:tdib8);
{  procedure BoutTriangle;
  Begin
    asm
      push ebx
      push edi
      push esi

      mov eax,TriNbp
      or eax,eax
      jns @Pos
      neg TriNbp
      neg TriINbp
      std
@pos:

@B1:
      mov edi,TriOff
      shr edi,12
      add edi,TriData

      mov ecx,TriNbp
      shr ecx,12

      mov eax,TriMx2
      sub eax,TriMx1
      cdq
      idiv ecx
      mov ebx,eax   // pas en x

      mov eax,TriMy2
      sub eax,TriMy1
      cdq
      idiv ecx
      mov esi,eax   // pas en y

      mov eax,ecx
      shl eax,8

      mov ecx,TriMy1
      mov edx,TriMx1
      push ebp
      mov ebp,ebx
      // edi:ecran , bp:pas x , esi:pas y , ah:nbp , ecx:y , edx:x , ah:nbpixels
@b2:  mov ebx,ecx
      mov bl,dh
      add ebx,TriTex
      mov al,[ebx]
      add edx,ebp
      add ecx,esi
      StosB
      dec ah
      jnz @b2
      pop ebp

      dec TriH
      jnz @B1

      mov edi,TriIOff
      add TriOff,edi

      mov edi,TriINbp
      add TriNbp,edi

      mov edi,TriIMy1
      add TriMy1,edi
      mov edi,TriIMx1
      add TriMx1,edi
      mov edi,TriIMy2
      add TriMy2,edi
      mov edi,TriIMx2
      add TriMx2,edi

      cld
      pop esi
      pop edi
      pop ebx
    end;
  End;

  procedure exch(var i,j:integer);
  var tmp:integer;
  Begin
    tmp:=i;
    i:=j;
    j:=tmp;
  End;

var p1,p2,p3,pt,mp1,mp2,mp3,mp:integer;
    inc1,inc2,inc3,incm1,incm2,incm3:integer;
    x1,x2,x3,y1,y2,y3,mx1,my1,mx2,my2,mx3,my3:integer;
Begin
  x1:=round(pt1.xr);
  x2:=round(pt2.xr);
  x3:=round(pt3.xr);
  y1:=round(pt1.yr);
  y2:=round(pt2.yr);
  y3:=round(pt3.yr);
  mx1:=round(pt1.tx*(dib.width-1));
  mx2:=round(pt2.tx*(dib.width-1));
  mx3:=round(pt3.tx*(dib.width-1));
  my1:=round(pt1.ty*(dib.Height-1));
  my2:=round(pt2.ty*(dib.Height-1));
  my3:=round(pt3.ty*(dib.Height-1));
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
    exch(my1,my2);
    exch(mx1,mx2);
  End;
  if y2>y3 then
  Begin
    exch(y2,y3);
    exch(x2,x3);
    exch(my2,my3);
    exch(mx2,mx3);
  End;
  if y1>y2 then
  Begin
    exch(y1,y2);
    exch(x1,x2);
    exch(my1,my2);
    exch(mx1,mx2);
  End;
  p1:=(y1*Width+x1) shl 12;
  p2:=(y2*Width+x2) shl 12;
  p3:=(y3*Width+x3) shl 12;
  mp1:=(my1*dib.width+mx1) shl 12;
  mp2:=(my2*dib.width+mx2) shl 12;
  mp3:=(my3*dib.width+mx3) shl 12;
  if y1=y2 then
    if y1=y3 then
     Begin
       BoutTriangle(pxls,0,0,p3,p1,mp3,mp1,0,0,dib.Pxls,1);
       BoutTriangle(pxls,0,0,p2,p1,mp2,mp1,0,0,dib.Pxls,1)
     end
    else
    Begin
      inc1:=(p3-p1) div (y3-y1);
      inc2:=(p3-p2) div (y3-y2);
      incm1:=(mp3-mp1) div (y3-y1);
      incm2:=(mp3-mp2) div (y3-y1);
      BoutTriangle(pxls,inc1,inc2,p1,p2,mp1,mp2,incm1,incm2,dib.Pxls,y3-y1+1)
    End
  else
  begin
    inc1:=(p3-p1) div (y3-y1);
    inc2:=(p2-p1) div (y2-y1);
    incm1:=(mp3-mp1) div (y3-y1);
    incm2:=(mp2-mp1) div (y2-y1);
    pt:=p1;
    mp:=mp1;
    BoutTriangle(Pxls,inc1,inc2,p1,pt,mp1,mp,incm1,incm2,dib.pxls,y2-y1);
    if y2<>y3 then
    begin
      inc3:=(p3-p2) div (y3-y2);
      incm3:=(mp3-mp2) div (y3-y2);
      BoutTriangle(pxls,inc1,inc3,p1,p2,mp1,mp2,incm1,incm3,dib.pxls,y3-y2+1);
    end;
  end;}
begin
End;


procedure TDIB8.clear(color:byte);
var isize,p:integer;
begin
  isize:=bi^.bmiHeader.biSizeImage;
  p:=integer(pxls);
  asm
    push edi
    mov edi,p
    mov ecx,isize
    mov al,color
    mov ah,al
    shl eax,16
    mov al,color
    mov ah,al
    shr ecx,2
    rep stosD
    pop edi
  end;
end;

Function TDIB8.LoadFromFile(FileName:TFileName):boolean;
var f:THandle;
    bfh:TBitmapfileHeader;
    bih:TBitmapInfoHeader;
    res:dword;
    i: integer;
    p: PByte;
begin
  result:=false;
  f:=CreateFile(Pchar(FileName),GENERIC_READ,FILE_SHARE_READ,nil,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
  if f=0 then
    exit;
  try
    readFile(f,bfh,sizeof(bfh),res,nil);
    if res<>sizeof(bfh) then
      exit;
    if bfh.bfType<>$4D42 then
      exit;
    ReadFile(f,bih,sizeof(bih),res,nil);
    if res<>sizeof(bih) then
      exit;
    bi^.bmiHeader:=bih;
    Width:=(bi^.bmiHeader.biWidth+3) and $fffffffC;
    Height:=bi^.bmiHeader.biHeight;
    readFile(f,bi^.bmiColors,256*sizeof(TRGBQUAD),res,nil);
    if res<>256*sizeof(TRGBQUAD) then
      exit;
    p:= pxls;
    for i:=0 to height-1 do
    Begin
      ReadFile(f,p^,bi^.bmiHeader.biWidth,res,nil);
      inc(p, width);
    end;
    result:=true;
  finally
    CloseHandle(f);
  end;
end;

function TDIB8.DrawToDC(DC:Hdc; x,y:integer):integer;
begin
  result:=Windows.SetDIBitsToDevice(DC,x,y,IWidth,IHeight,0,0,0,IHeight,Pxls,bi^,DIB_RGB_COLORS)
end;


constructor TTransMat.create;
Begin
  inherited create;
  identity;
End;

procedure TTransMat.rotate(x,y,z:single);
var a,b,c,d,e,f:extended;
    t11,t12,t13,t21,t22,t23,t31,t32,t33:single;
    t1,t2:single;
Begin
  sincos(x,b,a);
  sincos(y,d,c);
  sincos(z,f,e);
  t11:=a*c;
  t12:=-b*e-a*d*f;
  t13:=b*f-a*d*e;
  t21:=b*c;
  t22:=a*e-b*d*f;
  t23:=-a*f-b*d*e;
  t31:=d;
  t32:=c*f;
  t33:=c*e;

  t1:=matrix[0,0]*t11+matrix[0,1]*t21+matrix[0,2]*t31;
  t2:=matrix[0,0]*t12+matrix[0,1]*t22+matrix[0,2]*t32;
  matrix[0,2]:=matrix[0,0]*t13+matrix[0,1]*t23+matrix[0,2]*t33;
  matrix[0,1]:=t2;
  matrix[0,0]:=t1;
  t1:=matrix[1,0]*t11+matrix[1,1]*t21+matrix[1,2]*t31;
  t2:=matrix[1,0]*t12+matrix[1,1]*t22+matrix[1,2]*t32;
  matrix[1,2]:=matrix[1,0]*t13+matrix[1,1]*t23+matrix[1,2]*t33;
  matrix[1,1]:=t2;
  matrix[1,0]:=t1;
  t1:=matrix[2,0]*t11+matrix[2,1]*t21+matrix[2,2]*t31;
  t2:=matrix[2,0]*t12+matrix[2,1]*t22+matrix[2,2]*t32;
  matrix[2,2]:=matrix[2,0]*t13+matrix[2,1]*t23+matrix[2,2]*t33;
  matrix[2,1]:=t2;
  matrix[2,0]:=t1;
  t1:=matrix[3,0]*t11+matrix[3,1]*t21+matrix[3,2]*t31;
  t2:=matrix[3,0]*t12+matrix[3,1]*t22+matrix[3,2]*t32;
  matrix[3,2]:=matrix[3,0]*t13+matrix[3,1]*t23+matrix[3,2]*t33;
  matrix[3,1]:=t2;
  matrix[3,0]:=t1;
End;

procedure TTransMat.translate(x,y,z:single);
Begin
  matrix[3,0]:=matrix[3,0]+x;
  matrix[3,1]:=matrix[3,1]+y;
  matrix[3,2]:=matrix[3,2]+z;
End;

procedure TTransMat.scale(x,y,z:single);
Begin
  matrix[0,0]:=matrix[0,0]*x; matrix[1,0]:=matrix[1,0]*x; matrix[2,0]:=matrix[2,0]*x; matrix[3,0]:=matrix[3,0]*x;
  matrix[0,1]:=matrix[0,1]*y; matrix[1,1]:=matrix[1,1]*y; matrix[2,1]:=matrix[2,1]*y; matrix[3,1]:=matrix[3,1]*y;
  matrix[0,2]:=matrix[0,2]*z; matrix[1,2]:=matrix[1,2]*z; matrix[2,2]:=matrix[2,2]*z; matrix[3,2]:=matrix[3,2]*z;
End;

procedure TTransMat.identity;
var i,j:integer;
Begin
  for i:=0 to 3 do
    for j:=0 to 3 do
      if i=j then
        matrix[i,j]:=1
      else
        matrix[i,j]:=0;
End;

procedure TTransMat.multiplyPointsPos(var v1:array of TPoints3d);
  procedure calc(p1,p2:pointer; count:integer; a,b,c,d:single);
  asm
    push esi
    push edi
    mov esi,p1
    mov edi,p2
    fld d
    fld c
    fld b
    fld a
@b1:
    fld st(0)
    fmul dword ptr [esi]
    fld st(2)
    fmul dword ptr [esi+4]
    add esi,60
    faddp st(1),st(0)
    fld st(3)
    fmul dword ptr [esi+8-60]
    add edi,60
    faddp st(1),st
    dec count
    fadd st,st(4)
    fstp dword ptr [edi-60]
    jnz @b1
    pop edi
    pop esi
    finit
  end;
var count : integer;
Begin
  count:=high(v1);
  calc(@v1,@(v1[0].xr),count+1,matrix[0,0],matrix[1,0],matrix[2,0],matrix[3,0]);
  calc(@v1,@(v1[0].yr),count+1,matrix[0,1],matrix[1,1],matrix[2,1],matrix[3,1]);
  calc(@v1,@(v1[0].zr),count+1,matrix[0,2],matrix[1,2],matrix[2,2],matrix[3,2]);
End;

procedure TTransMat.multiplyPointsNorm(var v1:array of TPoints3d);
  procedure calc2(p1,p2:pointer; count:integer; a,b,c:double);
  asm
    push esi
    fld c
    push edi
    fld b
    fld a
    mov esi,p1
    mov edi,p2
@b1:
    fld st(0)
    fmul dword ptr tpoints3d [esi+16]
    fld st(2)
    fmul dword ptr tpoints3d [esi+20]
    add esi,60
    faddp st(1),st
    fld st(3)
    fmul dword ptr tpoints3d [esi+24-60]
    add edi,60
    faddp st(1),st
    dec count
    fstp dword ptr [edi-60]
    jnz @b1
    pop edi
    pop esi
    finit
  end;
var count : integer;
Begin
  count:=high(v1);
  calc2(@v1,@(v1[0].nxr),count+1,matrix[0,0],matrix[1,0],matrix[2,0]);
  calc2(@v1,@(v1[0].nyr),count+1,matrix[0,1],matrix[1,1],matrix[2,1]);
  calc2(@v1,@(v1[0].nzr),count+1,matrix[0,2],matrix[1,2],matrix[2,2]);
End;

procedure TTransMat.multiplyFaceNorm(var v1:array of TFace3d);
  procedure calc2(p1,p2:pointer; count:integer; a,b,c:double);
  asm
    push esi
    fld c
    push edi
    fld b
    fld a
    mov esi,p1
    mov edi,p2
@b1:
    fld st(0)
    fmul dword ptr [esi+12]
    fld st(2)
    fmul dword ptr [esi+16]
    add edi,40
    faddp st(1),st
    fld st(3)
    fmul dword ptr [esi+20]
    add esi,40
    faddp st(1),st
    dec count
    fstp dword ptr [edi-40]
    jnz @b1
    pop edi
    pop esi
    finit
  end;
var count : integer;
Begin
  count:=high(v1);
  calc2(@v1[0],@(v1[0].nxr),count+1,matrix[0,0],matrix[1,0],matrix[2,0]);
  calc2(@v1[0],@(v1[0].nyr),count+1,matrix[0,1],matrix[1,1],matrix[2,1]);
  calc2(@v1[0],@(v1[0].nzr),count+1,matrix[0,2],matrix[1,2],matrix[2,2]);
End;

const
  c4 : double = 4;
function TDib8.mandel(x1,y1,x2,y2:double):integer;
var ypas,xpas:double;
    prt : pointer;
    xw, yw:integer;
begin
  xw:=width;
  yw:=height;
  ypas:=(y2-y1)/yw;
  xpas:=(x2-x1)/xw;
  prt := pxls;
  result:=0;
  asm
    push esi
    push ebx
    mov esi,prt
    mov edx,yw
    fld c4
    fld y1
@b1:
     mov ecx,xw
     fld x1
@b2:
     mov ebx,100
     fld st(1)
     fld st(1)  // 4 y x iy ix     dib41
@b3:
//     inc result
     fld st(0)         // 4 y x iy      ix      ix
     fmul st(0),st(0)  // 4 y x iy      ix      ix2
     fld st(2)         // 4 y x iy      ix      ix2      iy
     fmul st(0),st(0)  // 4 y x iy      ix      ix2      iy2
     fxch st(1)        // 4 y x iy      ix      iy2      ix2
     fld st(0)         // 4 y x iy      ix      iy2      ix2     ix2
     fsub st(0),st(2)  // 4 y x iy      ix      iy2      ix2     ix2-iy2
     fxch st(2)        // 4 y x iy      ix      ix2-iy2  ix2     iy2
     faddp st(1),st(0) // 4 y x iy      ix      ix2-iy2  ix2+iy2
     fcomp st(6)       // 4 y x iy      ix      ix2-iy2
     fxch st(1)        // 4 y x iy      ix2-iy2 ix
     fmul st(0),st(2)  // 4 y x iy      ix2-iy2 ix*iy
     fnstsw ax
     fadd st(0),st(0)  // 4 y x iy      ix2-iy2 2*iy*ix
     and ah,1
     fstp st(2)        // 4 y x 2*iy*ix ix2-iy2
     jz @fin
     fadd st(0),st(2)  // 4 y x 2*iy*ix ix'
     fxch st(1)        // 4 y x ix'     2*iy*ix
     fadd st(0),st(3)  // 4 y x ix'     iy'
     dec ebx
     fxch st(1)        // 4 y x iy'     ix'
     jnz @b3
@fin:
     mov eax,110
     fstp st(0)        // y x iy'
     sub eax,ebx
     fstp st(0)        // 4 y x
     mov [esi],al
     inc esi
     dec ecx
     fadd xpas
     jnz @b2

    fstp st(0)
    dec edx
    fadd Ypas
    jnz @b1

    fstp st(0)
    fstp st(0)
    pop ebx
    pop esi
  end;
end;

function TDib8.mandel2(x1,y1,x2,y2:double; b:integer):integer;
var ypas,xpas:double;
    prt : pointer;
    xw, yw:integer;
begin
  xw:=width;
  yw:=height;
  ypas:=(y2-y1)/yw;
  xpas:=(x2-x1)/xw;
  prt := pxls;
  result:=0;
  asm
    push esi
    push ebx
    mov esi,prt
    mov edx,yw
    fld c4
    fld y1
@b1:
     mov ecx,xw
     fld x1
@b2:
     mov ebx,b
     fld st(1)
     fld st(1)  // 4 y x iy ix     dib41
@b3:
//     inc result
     fld st(0)         // 4 y x iy      ix      ix
     fmul st(0),st(0)  // 4 y x iy      ix      ix2
     fld st(2)         // 4 y x iy      ix      ix2      iy
     fmul st(0),st(0)  // 4 y x iy      ix      ix2      iy2
     fxch st(1)        // 4 y x iy      ix      iy2      ix2
     fld st(0)         // 4 y x iy      ix      iy2      ix2     ix2
     fsub st(0),st(2)  // 4 y x iy      ix      iy2      ix2     ix2-iy2
     fxch st(2)        // 4 y x iy      ix      ix2-iy2  ix2     iy2
     faddp st(1),st(0) // 4 y x iy      ix      ix2-iy2  ix2+iy2
     fcomp st(6)       // 4 y x iy      ix      ix2-iy2
     fxch st(1)        // 4 y x iy      ix2-iy2 ix
     fmul st(0),st(2)  // 4 y x iy      ix2-iy2 ix*iy
     fnstsw ax
     fadd st(0),st(0)  // 4 y x iy      ix2-iy2 2*iy*ix
     and ah,1
     fstp st(2)        // 4 y x 2*iy*ix ix2-iy2
     jz @fin
     fadd st(0),st(2)  // 4 y x 2*iy*ix ix'
     fxch st(1)        // 4 y x ix'     2*iy*ix
     fadd st(0),st(3)  // 4 y x ix'     iy'
     dec ebx
     fxch st(1)        // 4 y x iy'     ix'
     jnz @b3
@fin:
     mov eax,b
     add eax,10
     fstp st(0)        // y x
     sub eax,ebx
     fstp st(0)        // y x
     mov [esi],al
     inc esi
     dec ecx
     fadd xpas
     jnz @b2

    fstp st(0)
    dec edx
    fadd Ypas
    jnz @b1

    fstp st(0)
    fstp st(0)
    pop ebx
    pop esi
  end;
end;


initialization
  IntegerTabUse:=0;
finalization
  reallocMem(IntegerTab,0);
end.


