program Debug4x;

//  Changes marked with WGG are copyright 2002, William G. Graves

//  In project options, define UseConn4x for ConnectivityKit

uses
  Forms,
  classes,
  Controls,
  windows,
  registry,
  WindowsPosSave in 'Components\CDBShort\WindowsPosSave.pas',
  CDBShUtils in 'Components\CDBShort\CDBShUtils.pas',
  utils2 in 'Components\CDBUtils\utils2.pas',
  MouseScroolControl in 'Components\CDBUtils\MouseScroolControl.pas',
  SynEdit in 'Synedit1.15\Source\SynEdit.pas',
  fmMainform in 'fmMainForm.pas' {frmMainForm},
  fmDebugEMU in 'fmDebugEMU.pas' {frmDebugEMU},
  Debug in 'Debug.pas',
  fmRamView in 'fmRamView.pas' {frmRamView},
  Desasm in 'Desasm.pas',
  Search in 'Search.pas',
  fmDebugRPL in 'fmDebugRPL.pas' {frmDebugRPL},
  fmBreakPointChoice in 'fmBreakPointChoice.pas' {frmBreakPointChoice},
  fmProgress in 'fmProgress.pas' {frmProgress},
  fmCPUCompare in 'fmCPUCompare.pas' {frmCPUCompare},
  fmEntryPoints in 'fmEntryPoints.pas' {frmEntryPoints},
  RPLDesasm in 'RPLDesasm.pas',
  fmMapLRFiles in 'fmMapLRFiles.pas' {frmMapLRFile},
  fmSourceEdit in 'fmSourceEdit.pas' {frmSourceEdit},
  LRWorking in 'LRWorking.pas',
  fmAbout in 'fmAbout.pas' {frmAbout},
  fmInformEdit in 'fmInformEdit.pas' {frmInformEdit},
  fmFullPageEdit in 'fmFullPageEdit.pas' {frmFullPageEdit},
  fmProject in 'fmProject.pas' {frmProject},
  HPToolsdll in 'HPToolsdll.pas',
  fmTextEval in 'fmTextEval.pas' {frmTextEval},
  dlgSearchText in 'dlgSearchText.pas' {dlgSearchText},
  dlgReplaceText in 'dlgReplaceText.pas' {dlgReplaceText},
  dlgConfirmReplace in 'dlgConfirmReplace.pas' {dlgConfirmReplace},
  dlgPageSetup in 'dlgPageSetup.pas' {dlgPageSetup},
  dlgPrintPreview in 'dlgPrintPreview.pas' {dlgPrintPreview},
  RegistryValues in 'RegistryValues.pas',
  emu48dll in 'emu48dll.pas',
  KeyEdit in 'KeyEdit.pas' {Form2},
  UHelpKeys in 'UHelpKeys.pas' {HelpKeys};

{$R *.RES}
{$R FileIcons.RES}

procedure DoRegFileTypes;
  //wgg Setup icons and file types
  procedure RegFileExtension(extension: string; typename: string; description: string;
                             icon: string; DoOpen: boolean; DoPrint: boolean);
  begin
    with TRegistry.Create do
    begin
      RootKey:= HKEY_CLASSES_ROOT;
      if KeyExists(extension) then
        if Application.MessageBox(pchar('The file type *'+extension+' already has an association in the registry.'#$0D
                                       +'Would you like this changed to Debug4x?'),
                                  pchar(extension+' File Type'), MB_YESNO) <> IDYES then
        begin
          free;
          exit;
        end;
      OpenKey('\'+extension, true);
      WriteString('', typename);            // default value
      OpenKey('\'+typename+'\DefaultIcon', true);
      WriteString('', '"'+ApplicationPath + icon+'"');
      if DoOpen then
      begin
        OpenKey('\'+typename+'\shell\open\command', true);
        WriteString('', '"'+ApplicationPath + '\Debug4x.exe" "%1"');
      end;
      if DoPrint then
      begin
        OpenKey('\'+typename+'\shell\print\command', true);
        WriteString('', '"Notepad.exe" /p "%1"');
      end;
      free;
    end;
  end;

begin
   // Create file-extension types
   {RegFileExtension('.hpp', 'hpp_auto_file', 'HP Development Kit Project File',
                    '\Include\Project.ico', true, false);
   RegFileExtension('.hp',  'hp_auto_file',  'HP Calculator Program File',
                    'Debug4x.exe', false, false);  }
   RegFileExtension('.s', 'sfile', 'HP Calculator Source File',
                    '\Include\sourceFile.ico', true, true);  
   RegFileExtension('.h', 'hfile', 'HP Calculator Header File',
                    '\Include\headerFile.ico', true, true);
end;

// wgg - check for registry initialized, if not initialize
// Install sets up the current user
// if there are other users, their registry will not be set!
procedure InitRegistry;
var
  r: TRegIniFile;
  rplKey, tkey, debRoot: string;

  Procedure DebugSetOne(key:string; prefix: string; x, y, w, h, v: integer);
  var
    tkey: string;
  begin
     tkey:= key;
     if r.KeyExists(tkey + prefix) then   exit;  // wgg if key already there, do not disturb its values
     r.WriteInteger(tkey, prefix + 'Top', y);
     r.WriteInteger(tkey, prefix + 'Left', x);
     r.WriteInteger(tkey, prefix + 'Visible', v);
     if (h <> 0) And (w <> 0) then                   // not all windows can grow
     begin
       r.WriteInteger(tkey, prefix + 'Height', h);
       r.WriteInteger(tkey, prefix + 'Width', w);
     end;
  end;

begin
  r:= TRegIniFile.Create(RegistryKey);
  rplKey:= 'RPLDebug';
  if Not r.KeyExists('ASMDebug') then
  begin                 // do complete registry initialize
     // Values are:                left, top, wid, hgt,  visible
     DebugSetOne('Main',        '',   1,   1,   0,   0,  1);   // wid is 219, hgt is 109
     DebugSetOne('Project',     '',   1, 110,   0,   0,  1);   // wid is 341, hgt is 254
     DebugSetOne('SourceEdit',  '', 338,   1, 716, 600,  1);
     DebugSetOne('InformEdit',  '',  50, 200,   0,   0,  0);

     DebugSetOne(rplKey,        '',   1, 362, 353, 557,  1);
     DebugSetOne('ASMDebug',    '',  50, 363,   0,   0,  0);

     DebugSetOne('BreakPoints', '', 238,  30,   0,   0,  0);
     DebugSetOne('EntryPoints', '', 300, 370,   0,   0,  0);
     DebugSetOne('FileMap',     '', 300, 380, 444, 200,  0);
     DebugSetOne('MemoryMap',   '', 300, 380, 390, 500,  0);

{$IFDEF UseConn4x}
     DebugSetOne('Connectivity',   '', 325, 190, 540, 390,  0);
{$ENDIF}

     // Values are:                               left, top, wid, hgt,  visible
     DebugSetOne(rplKey, 'BitMaps',                  3, 347, 172,  75,  1);
     DebugSetOne(rplKey, 'Control',                208,   3, 126, 208,  1);
     DebugSetOne(rplKey, 'FlashPointers',          847,  80, 158,  80,  1);
     DebugSetOne(rplKey, 'Home_Directory',           0, 426, 345, 143,  1);
     DebugSetOne(rplKey, 'Last_Instructions',      209, 215, 129, 143,  1);
     DebugSetOne(rplKey, 'Local_Variables',          2, 118, 203, 129,  1);
     DebugSetOne(rplKey, 'Arrays',                 413, 425, 232,  96,  1);
     DebugSetOne(rplKey, 'Memory_Map',             650,   4, 400, 174,  1);
     DebugSetOne(rplKey, 'Memory_Test',            642, 188, 193,  52,  0);
     DebugSetOne(rplKey, 'Profiler',               431, 286, 283, 120,  1);
     DebugSetOne(rplKey, 'Return_Stack',             3, 247, 202,  99,  1);
     DebugSetOne(rplKey, 'Rpl_Stack',                0,   0, 207, 117,  1);
     DebugSetOne(rplKey, 'Step_Over_Entry_Points', 562, 285,  89, 139,  0);  // wgg this window does not work as described, just hide it!
     DebugSetOne(rplKey, 'Temporary_Objects',      427,   1, 262, 281,  1);
     DebugSetOne(rplKey, 'Virtual_Stack',          679, 301,  97, 206,  1);

     // Do the minimum registry keys needed for EMU48
     debRoot:= '\'+RegistryKey+'\'+'Emu48';
     tkey:= debRoot + '\Files';
     r.WriteString(tkey, 'Emu48Directory', ApplicationPath + '\Emu');
     r.OpenKey(tkey, True);
     TRegistry(r).writeInteger('AutoSaveOnExit', 0);   // wgg need to get a DWORD into registry
     tkey:= debRoot + '\KML';
     r.OpenKey(tkey, True);
     TRegistry(r).writeInteger('AlwaysDisplayLog', 0); // wgg need to get a DWORD into registry
     tkey:= debRoot + '\Port2';
     r.OpenKey(tkey, True);
     TRegistry(r).WriteString('Filename', 'Default.bin'); 

     DoRegFileTypes;
  end;
end;

Procedure GiveCompletion(c: TComponent);
var
  i: integer;
begin
  if assigned(c) then
  begin
    // wgg if more help (F1) is available, prefix a clue for user
    if (c is TControl) And ((c as Tcontrol).helpcontext>1) And
        ((c as Tcontrol).hint<>'') And (copy((c as Tcontrol).hint,1,1) <> '?') then
      if ((c as Tcontrol).helpcontext Mod 10000)>=5000 then
        (c as Tcontrol).hint:= '?* '+(c as Tcontrol).hint
      else (c as Tcontrol).hint:= '? '+(c as Tcontrol).hint;

    if c is TSynEdit then
    Begin
      frmSourceEdit.SetEditProps(c as TSynEdit);
    {  frmMainForm.SynCompletionProposal1.AddEditor(c as TSynEdit);
      frmMainForm.SynAutoComplete1.Editor:= (c as TSynEdit); }      // wgg there are many such editors
    end;

    for i:= 0 to c.componentCount-1 do
      GiveCompletion(c.Components[i]);
    end;
end;

begin
 { Application.Initialize; }
  Application.Title := 'Debugger';
  Application.HelpFile := 'Debug4x.chm';
  Application.ShowHint:= True;
  InitRegistry;                                                 // wgg
  Application.CreateForm(TfrmMainForm, frmMainForm);
  Application.CreateForm(TfrmDebugEMU, frmDebugEMU);
  Application.CreateForm(TfrmRamView, frmRamView);
  Application.CreateForm(TfrmDebugRPL, frmDebugRPL);
  Application.CreateForm(TfrmBreakPointChoice, frmBreakPointChoice);
  Application.CreateForm(TfrmCPUCompare, frmCPUCompare);
  Application.CreateForm(TfrmEntryPoints, frmEntryPoints);
  Application.CreateForm(TfrmSourceEdit, frmSourceEdit);
  Application.CreateForm(TfrmAbout, frmAbout);
  Application.CreateForm(TfrmInformEdit, frmInformEdit);
  Application.CreateForm(TfrmFullPageEdit, frmFullPageEdit);
  Application.CreateForm(TfrmProject, frmProject);
  Application.CreateForm(TfrmTextEval, frmTextEval);
  Application.CreateForm(TfrmMapLRFiles, frmMapLRFiles);
  Application.CreateForm(TPrintPreviewDlg, PrintPreviewDlg);
  Application.CreateForm(TPageSetupDlg, PageSetupDlg);
  Application.CreateForm(THelpKeys, HelpKeys);
  GiveCompletion(Application);
  // form's BorderIcons, biHelp must be true for pop-up help
  GiveCompletion(Application);
 // form's BorderIcons, biHelp must be true for pop-up help
  Application.OnHelp := frmMainForm.ApplicationOnHelp;                // wgg hook up the HTML help syste
  Application.Run;
end.
