{
  This units is a RPL decompiler.
  It allows you do decompile an RPL object located
  at a given address.
}
unit RPLDesasm;

interface
Uses lrworking, sysutils, CDBShUtils, FmProject, Debug;

type
  TReadInt64= Function (adr: integer): int64 of object;
  TReadInt= Function (adr: integer): integer of object;
  TAddBitmap= function (const s: string):string of object;

Type
  // This enumeration is a list of all the object types in the HP49
  // Note: the order must be the same that the PrologsNames const.
  // wgg keep HP49 specific items at the end so we can prune them for HP48 use
  TPrologs =( DOACPTR  ,
              DOARRY   , //strobj
              DOBAK    , //strobj
              DOBINT   ,
              DOCHAR   ,
              DOCMP    ,
              DOCODE   , //strobj
              DOCOL    ,
              DOCSTR   , //strobj
              DOECMP   , //strobj
              DOEREL   , //strobj
              DOEXT    ,
              DOEXT0   ,
              DOEXT2   , //strobj
              DOEXT3   , //strobj
              DOEXT4   , //strobj
              DOGROB   , //strobj
              DOHXS    , //strobj
              DOIDNT   ,
              DOLAM    ,
              DOLIB    , //strobj
              DOLIST   ,
              DOLNKARRY, //strobj
              DOREAL   ,
              DOROMP   ,
              DORRP    ,
              DOSYMB   ,
              DOTAG    ,
              SEMI     ,
              // wgg HP49 specific items
              DOAPLET  , //strobj
              DOFLASHP ,
        {     DOFONT   , //strobj      }                                        // wgg cannot find in suprom49.a
              DOINT    ,
              DOLNGCMP ,
              DOLNGREAL,
              DOMATRIX ,
              DOMINIFONT );  //strobj

  // This type intent to save the value of every prolog in the HP49.
  TPrologsList = array [TPrologs] of integer;

  // this function allow the decompilation of the object at a specified address
  // adr: the adr of the object to decompile (at the end, it's the adr of the end of the object)
  // the 2 next parameters intent to receive a function alloging to read from the memory
  // Prolog intent to receive a list of the value of each prolog
  // the function returns a string representing the object
  Function DecompObj(Var adr: integer; ReadInteger: TReadInt; ReadInt64: TReadInt64; const prolog: TPrologsList; addbitmap, AddMatrices: TAddBitmap): String;
  // this initialize a TPrologList structure.
  Procedure InitPrologues(var prolog: TPrologsList);
  Function CheckFlashP(adr: integer; ReadInteger: TReadInt; prolog: TPrologsList; bds: array of integer): Boolean;
  Procedure InitBads(var bds: array of integer);
  function dreal(e: integer; v: int64): string;

implementation

const
  // This is the list of the prolog constants. Tis values are used to retrive the
  // numerical value from the .lr files.
  // Note: the order must be the same that the TPrologs type.
  PrologsNames : array [TPrologs] of string =(
              'DOACPTR',
              'DOARRY',
              'DOBAK',
              'DOBINT',
              'DOCHAR',
              'DOCMP',
              'DOCODE',
              'DOCOL',
              'DOCSTR',
              'DOECMP',
              'DOEREL',
              'DOEXT',
              'DOEXT0',
              'DOEXT2',
              'DOEXT3',
              'DOEXT4',
              'DOGROB',
              'DOHXS',
              'DOIDNT',
              'DOLAM',
              'DOLIB',
              'DOLIST',
              'DOLNKARRY',
              'DOREAL',
              'DOROMP',
              'DORRP',
              'DOSYMB',
              'DOTAG',
              'SEMI',
              // wgg HP49 specific items, if this list changes patch InitPrologues below
              'DOAPLET',
              'DOFLASHP',
            { 'DOFONT',  }                                                      // wgg cannot find in suprom49.a
              'DOINT',                                                          
              'DOLNGCMP',
              'DOLNGREAL',
              'DOMATRIX',
              'DOMINIFONT');

function dreal(e: integer; v: int64): string;
Begin
  result:= '';
  e:= (e and $f)+((e shr 4) and $f)*10+((e shr 8) and $f)*100+((e shr 12) and $f)*1000+((e shr 16) and $f)*10000;
  if e>=50000 then e:= -(100000-e);
  if v shr 60 <>0 then result:= result+'-';
  if (e>=0) and (e<10) then
  begin
    e:= e+1;
    while e<>0 do begin result:= result+inttostr((v shr 56) and $f); v:= v shl 4; dec(e); end
  end
  else
    if (e<0) and (e>-10) then
    begin
      result:= result+copy('0.00000000000', 1, 1-e);
      e:= 0;
    end else begin
      result:= result+inttostr((v shr 56) and $f)+'.';
      v:= v shl 4;
    end;
  if pos('.', result)=0 then result:= result+'.';
  result:= result+inttohex(v and $fffffffffffffff, 15);
  while result[length(result)]='0' do result:= copy(result, 1, length(result)-1);
  if e <> 0 then
    result:= result+'E'+inttostr(e);
end;

// Init a TPrologsList struct retriving the values of each constant
Procedure InitPrologues(var prolog: TPrologsList);
var
  j: TPrologs;
begin
  for j:=Low(prolog) to High(Prolog) do
    if (frmProject.Calc48.Checked) and (j > SEMI) then        // wgg remove HP49 specific prologs
      prolog[j]:= High(Integer)
    else
      prolog[j]:= SymbolTable.ValueFromText(PrologsNames[j]) and $3FFFF;
    if prolog[DOFLASHP]<>High(Integer) then
      Debugger.FLASHPSEC:= Debugger.ReadInteger(prolog[DOFLASHP]+16)
    else
      Debugger.FLASHPSEC:= -1;
end;

// This is the main function doing the work.
Function DecompObj(Var adr: integer; ReadInteger: TReadInt; ReadInt64: TReadInt64;
                   const prolog: TPrologsList; addbitmap, AddMatrices: TAddBitmap): String;
var
  time: TDateTime;

  Function DecompObj2(var adr: integer): string; forward;
    // array decompilation (to be enhanced)
    Function ARRY(adr: integer; var s: string): integer;
    var
      ss: string;
      size, i: integer;
    Begin
      s := 'Array of ' + SymbolTable.GetFromAdr(ReadInteger(adr + 10));
      result:= adr+5+ReadInteger(adr+5);
      if assigned(AddMatrices) then
      begin
        ss:= '';
        size:= ReadInteger(adr+5)+5;
        i:=0;
        while i<=size do
        begin
          ss:= ss+inttohex2(readinteger(adr+i), 1);
          inc(i);
        end;
        s:= s+' '+AddMatrices(ss);
      end;
    end;
    // binary integer decompilation
    Function BINT(adr: integer; var s: string): integer;
    Begin
      s:= '# '+inttohex2(ReadInteger(adr+5), 5);
      result:= adr+10;
    end;
    // character decompilation (to be enhanced)
    Function CHAR(adr: integer; var s: string): integer;
    Begin
      s:= 'CHR '+inttohex2(ReadInteger(adr+5), 2);
      result:= adr+7;
    end;
    // normal complex decompilation (to be enhanced)
    Function CMP (adr: integer; var s: string): integer;
    Begin
      s:= 'Cmp '+inttohex2(ReadInteger(adr+5), 16)+ inttohex2(ReadInteger(adr+5+16), 16);
      result:= adr+5+16+16;
    end;
    // Assembly program decompilation (Need to add Hex)
    Function CODE(adr: integer; var s: string): integer;
    Begin
      s := 'Code';
      result:= adr+5+ReadInteger(adr+5);
    end;
    // Rpl program decompilation
    Function COL (adr: integer; var s: string): integer;
    Begin
      s := ':: ';
      inc(adr, 5);
      While ReadInteger(adr) <> Prolog[SEMI] Do
        s := s + DecompObj2(adr) + ' ';
      s := s + ' ;';
      result:= adr+5;
    end;
    // chr string decompilation (Need to be speeded up for long strings)
    Function CSTR(adr: integer; var s: string): integer;
    var
      i: integer;
      j: integer;
    Begin
      s := '"';
      i:= (ReadInteger(adr+5)-5) div 2;
//      if i>30 then i:= 30;
      for j:= 1 to i do
        if ReadInteger(adr+8+j*2)<32 then
          s:= s+#32
        else
          s:= s+chr(ReadInteger(adr+8+j*2));
      s:= s+'"';
      result:= adr+ ReadInteger(adr+5)+5
    end;
    // Units decompilation
    Function EXT (adr: integer; var s: string): integer;
    Begin
      adr := adr + 5;
      s := 'UNIT ';
      While ReadInteger(adr) <> Prolog[SEMI] Do
        s := s + DecompObj2(adr) + ' ';
      s := s + ' [;]';
      result:= adr+5;
    end;
    // library datas (hexa?)
    Function EXT0(adr: integer; var s: string): integer;
    Begin
      s := 'Lib Data';
      result:= adr+ ReadInteger(adr+5)+5;
    end;
    // Extended pointer. (Try to resolve the extended pointer if in ROM)
    Function ACPTR(adr: integer; var s: string): integer;
    Begin
      s := 'Ext pointer ' + IntToHex2(ReadInteger(adr + 5), 5) + ' ' + IntToHex2(ReadInteger(adr + 10), 5);
      result := adr + 15;
    end;
    // Extended objects. (Hexa?)
    Function EXT2(adr: integer; var s: string): integer;
    Begin
      s := 'Ext1';
      result:= adr+ ReadInteger(adr+5)+5;
    end;
    Function EXT3(adr: integer; var s: string): integer;
    Begin
      s := 'Ext2';
      result:= adr+ ReadInteger(adr+5)+5;
    end;
    Function EXT4(adr: integer; var s: string): integer;
    Begin
      s := 'Ext3';
      result:= adr+ ReadInteger(adr+5)+5;
    end;
    // Graphic. (Need a graphic editor)
    Function GROB(adr: integer; var s: string): integer;
    var
      ss: string;
      size, i: integer;
    Begin
      s := 'Grob';
      result:= adr+ReadInteger(adr+5)+5;
      if assigned(addbitmap) then
      begin
        ss:= '';
        size:= ReadInteger(adr+5)+5;
        i:=0;
        while i<=size do
        begin
          ss:= ss+inttohex2(readinteger(adr+i), 1);
          inc(i);
        end;
        s:= s+' '+addbitmap(ss);
      end;
    end;
    // binary integer
    Function HXS(adr: integer; var s: string): integer;
    var
      j: integer;
    Begin
      j:= ReadInteger(adr+5)-5;
      s := 'HXS '+inttohex2(j, 5)+' ';
      while j>0 do
      Begin
        s:= s+inttohex2(ReadInteger(adr+j+9) and $F, 1);
        dec(j);
      end;
      result:= adr+ReadInteger(adr+5)+5;
    end;
    // Identifier decompilation (need to avoid stranges chr like in DOCSTR need to create a ReadChr function)
    Function IDNT(adr: integer; var s: string): integer;
    var
      i: integer;
      j: integer;
    Begin
      s := 'IDNT ';
      i:= ReadInteger(adr+5) and $FF;
      for j:= 1 to i do
        s:= s+chr(ReadInteger(adr+5+j*2));
      result:= adr+i*2+7
    end;
    // Local Identifier decompilation (need to avoid stranges chr like in DOCSTR need to create a ReadChr function)
    Function LAM (adr: integer; var s: string): integer;
    var
      i: integer;
      j: integer;
    Begin
      s := 'LAM ';
      i:= ReadInteger(adr+5) and $FF;
      for j:= 1 to i do
        s:= s+chr(ReadInteger(adr+5+j*2));
      result:= adr+i*2+7
    end;
    // Library decompilation (need to put either the name and number or an hexa rep)
    Function LIB (adr: integer; var s: string): integer;
    Begin
      s := 'Lib';
      result:= adr+ReadInteger(adr+5)+5
    end;
    // Backup (need to decompile)
    Function BAK (adr: integer; var s: string): integer;
    Begin
      s := 'Bak';
      result:= adr+ReadInteger(adr+5)+5
    end;
    // List decompilation
    Function LIST(adr: integer; var s: string): integer;
    Begin
      s := '{ ';
      inc(adr, 5);
      While ReadInteger(adr) <> prolog[SEMI] Do
        s := s + DecompObj2(adr) + ' ';
      s := s + ' }';
      result:= adr+5;
    end;
    // real decompilation. (need to improve the decompilation)

    Function REAL(adr: integer; var s: string): integer;
    var
      i: integer;
    Begin
      i:= ReadInt64(adr+5) and $fff; if (i and $f00)>=$500 then i:= i+$99000;
      s:= '% '+dreal(i, (ReadInt64(adr+8) and $fffffffffffff) shl 12);
      result:= adr+5+16;
    end;
    // Extended real. (Need real decompilation)
    Function EREL(adr: integer; var s: string): integer;
    Begin
      s:= '%% '+dreal(ReadInt64(adr+5), ReadInt64(adr+10));
      result:= adr+5+16+5;
    end;

    // Extended complex. (Need real decompilation)
    Function ECMP(adr: integer; var s: string): integer;
    Begin
      s:= 'ECmp '+inttohex2(ReadInteger(adr+5), 5)+' '+ inttohex2(ReadInt64(adr+10), 16)+
                  inttohex2(ReadInteger(adr+5+16+1+5), 5)+' '+ inttohex2(ReadInt64(adr+5+16+1+5+5), 16);
      result:= adr+5+16+5+5+16;
    end;
    // Rompointer. (We should try to get the real value)
    Function ROMP(adr: integer; var s: string): integer;
    Begin
      s:= SymbolTable.GetFromAdr2(ReadInt64(adr + 5) And $FFFFFF);
      if (length(s)>1) and (s[1]='~') then
        s:= 'ROMPTR '+s
      else
        s := 'ROMPTR ' + IntToStr(ReadInteger(adr + 5) And $FFF) + ' ' + IntToStr(ReadInteger(adr + 5 + 3) And $FFF);
      result := adr + 5 + 6;
    end;
    // Linked array. (need real decompilation)
    Function LNKARRY(adr: integer; var s: string): integer;
    Begin
      s:= 'LnkArray';
      result:= adr+ReadInteger(adr+5)+5
    end;
    // symbolic object decompilation
    Function SYMB(adr: integer; var s: string): integer;
    Begin
      s := 'DOSYMB ';
      inc(adr, 5);
      While ReadInteger(adr) <> prolog[SEMI] Do
        s := s + DecompObj2(adr) + ' ';
      s := s + ' [;]';
      result:= adr+5;
    end;
    // tag object decompilation. (Need to use a getChr function)
    Function TAG(adr: integer; var s: string): integer;
    var
      i: integer;
      j: integer;
    Begin
      s := ':';
      i:= ReadInteger(adr+5) and $FF;
      for j:= 1 to i do
        s:= s+chr(ReadInteger(adr+5+j*2));
      s:= s+':';
      result:= adr+i*2+7;
      s:= s+DecompObj2(result);
    end;
    // Long integer decompilation. No syntax yet
    Function INT(adr: integer; var s: string): integer;
    var
      i, j: integer;
    Begin
      i:= ReadInteger(adr+5);
      if i>5 then
      Begin
        s:= '';
        if i<=6 then
          s:= s+'0';
        for j:=i downto 7 do
          s:=s+IntToHex2(ReadInteger(adr+3+j) and $f, 1);
        if (ReadInteger(adr+i+5-1) and $F)=9 then
          s:= 'ZINT -'+s
        else
          s:= 'ZINT '+s
      end;
      result:= adr+i+5
    end;
    // new Long real decompilation. No syntax yet
    Function LNGREAL(adr: integer; var s: string): integer;
    var
      s2: String;
    Begin
      s2:= 'LNReal ';
      adr:= INT(adr,s);
      s2:= s2+s+'.';
      result:= INT(adr-5,s);
      s:= s2+s;
    end;
    // new Long complex decompilation. No syntax yet
    Function LNGCMP(adr: integer; var s: string): integer;
    var
      s2: String;
    Begin
      s2:= 'LNReal ';
      adr:= LNGREAL(adr,s);
      s2:= s2+s+' ';
      result:= LNGREAL(adr-5,s);
      s:= s2+s;
    end;
    // flashpointer. (Need to put the symbol of exists)
    Function FLASHP(adr: integer; var s: string): integer;
    Begin
      s:= SymbolTable.GetFromAdr2(ReadInt64(adr + 5) And $FFFFFFF);
      if (length(s)>1) and (s[1]='^') then
        s:= 'FLASHPTR '+s
      else
        s := 'FLASHPTR ' + IntToStr(ReadInteger(adr + 5) And $FFF) + ' ' + IntToStr(ReadInteger(adr + 5 + 3) And $FFDF);
      result:= adr+5+3+4;
    end;
    // new symbolic matrix decompilation.
    Function MATRIX(adr: integer; var s: string): integer;
    Begin
      s:= 'DOMATRIX';         
      inc(adr, 5);
      While ReadInteger(adr) <> Prolog[SEMI] Do
        s := s + ' ' + DecompObj2(adr) ;          // wgg added space
      s:= s+'[;]';
      result:= adr+5;
    end;

  Function DecompObj2(var adr: integer): string;
  Var
    s: String;
    i: integer;
    j : TPrologs;
    j2: integer;
  Begin
    Try
      if now-time>1/24/3600*3 then
        raise exception.create('TimeOut');                                         
      result := '';
      // read the data at the address to decompile
      i := ReadInteger(adr);
      // search for it in the prolog list
      j:= Low(Prolog);
      while (j<=high(prolog)) and (prolog[j]<>i) do inc(j);
      // if it is a prolog, we are pointing directly on the object, decompile it
      case j of
        DOACPTR    : adr := ACPTR   (adr, s);
        DOARRY     : adr := ARRY    (adr, s);
        DOBAK      : adr := BAK     (adr, s);
        DOBINT     : adr := BINT    (adr, s);
        DOCHAR     : adr := CHAR    (adr, s);
        DOCMP      : adr := CMP     (adr, s);
        DOCODE     : adr := CODE    (adr, s);
        DOCOL      : adr := COL     (adr, s);
        DOCSTR     : adr := CSTR    (adr, s);
        DOECMP     : adr := ECMP    (adr, s);
        DOEREL     : adr := EREL    (adr, s);
        DOEXT      : adr := EXT     (adr, s);
        DOEXT0     : adr := EXT0    (adr, s);
        DOEXT2     : adr := EXT2    (adr, s);
        DOEXT3     : adr := EXT3    (adr, s);
        DOEXT4     : adr := EXT4    (adr, s);
        DOHXS      : adr := HXS     (adr, s);
        DOIDNT     : adr := IDNT    (adr, s);
        DOLAM      : adr := LAM     (adr, s);
        DOLIB      : adr := LIB     (adr, s);
        DOLIST     : adr := LIST    (adr, s);
        DOLNKARRY  : adr := LNKARRY (adr, s);
        DOREAL     : adr := REAL    (adr, s);
        DOROMP     : adr := ROMP    (adr, s);
        DOSYMB     : adr := SYMB    (adr, s);
        DOTAG      : adr := TAG     (adr, s);
        // HP49 specific items
        DOFLASHP   : adr := FLASHP  (adr, s);
        DOGROB     : adr := GROB    (adr, s);
        DOINT      : adr := INT     (adr, s);
        DOLNGCMP   : adr := LNGCMP  (adr, s);
        DOLNGREAL  : adr := LNGREAL (adr, s);
        DOMATRIX   : adr := MATRIX  (adr, s);

// To do
//        DOAPLET    : adr := APLET   (adr, s);
//        DOFONT     : adr := FONT    (adr, s);
//        DOMINIFONT : adr := MINIFONT(adr, s);
//        DORRP      : adr := RRP     (adr, s);

        // it is not a prolog.
        // if it is a pointer on a simple object, decompile this object,
        // else, if the pointer has a name, give the name
        // else, give the value of the pointer.
        else
          // skip the pointer
          adr:= adr+5;
          // read the pointed value
          j2:= ReadInteger(i);
          // look for it in the prolog list
          j:= Low(Prolog);
          while (j<=high(prolog)) and (prolog[j]<>j2) do inc(j);
          // if it is a simple object, decompile it
          case j of
            DOCHAR: CHAR(i,s);
            DOCMP : CMP (i,s);
            DOECMP: ECMP(i,s);
            DOEREL: EREL(i,s);
            DOCSTR: CSTR(i,s);
            DOHXS : HXS(i,s);
            DOIDNT: IDNT(i,s);
            DOLAM : LAM (i,s);
            DOREAL: REAL(i,s);
            DOTAG : TAG(i,s);
            // HP49 specific items
            DOBINT: BINT(i,s);
            DOFLASHP: FLASHP(i,s);
            DOINT : INT(i,s);
            DOLNGCMP: LNGCMP(i,s);
            DOLNGREAL: LNGREAL(i,s);
           else
              // if it is in the list, give the text value
              s:= SymbolTable.GetFromAdr(i);
              if s='' then
                // else, give a pointer
                s:= 'PTR '+inttohex2(i, 5);
          end;
      end;
    Except
      on e: exception do
        s := 'Error '+e.message;
    End;
    result:= s;
  End;
Begin
  time:= now;
  result:= DecompObj2(adr);
end;

// This is the main function doing the work.
const
  bads : array [0..13+4] of string =
  ('2''RCOLARPITE', 'COLACOLA', 'COLAITE', 'COLANOTcase', 'COLARPITE', 'COLASKIP',
   'COLA_EVAL', 'COLAcase', 'DROPCOLA', 'ONECOLA', 'RDROPCOLA', 'SWAPCOLA', 'SWAPCOLASTO', 'XYZ>ZCOLA',
   'ticR', '''R', 'RDROP', 'RSWAP');
Procedure InitBads(var bds: array of integer);
var
  i: integer;
begin
  for i:=0 to high(bads) do bds[i]:= SymbolTable.ValueFromText(bads[i]);
end;

Function CheckFlashP(adr: integer; ReadInteger: TReadInt; prolog: TPrologsList; bds: array of integer): Boolean;
var
  TOTEMPOB: integer;
  WaitForTOTEMPOB: boolean;

  Function DecompObj2(var adr: integer): boolean; forward;
  Function strObj(adr: integer): integer;
  Begin
    result:= adr+5+ReadInteger(adr+5);
  end;
  
  Function COL (var adr: integer): boolean;
  Begin
    result:= false;
    inc(adr, 5);
    While ReadInteger(adr) <> Prolog[SEMI] Do
    begin
      result:= DecompObj2(adr);
      if result then exit;
      if WaitForTOTEMPOB and (ReadInteger(adr)<>TOTEMPOB) then begin result:= true; exit; end;
    end;
    adr:= adr+5;
  end;

  Function DecompObj2(var adr: integer): boolean;
  Var
    i, k: integer;
    j : TPrologs;
  Begin
      result:= false;
      WaitForTOTEMPOB:= false;
      i := ReadInteger(adr);
      j:= Low(Prolog);
      while (j<=high(prolog)) and (prolog[j]<>i) do inc(j);
      case j of
        DOACPTR    : adr := adr + 15;
        DOARRY     : adr := strobj(adr);
        DOBAK      : adr := strobj(adr);
        DOBINT     : adr := adr+10;
        DOCHAR     : adr := adr+7;
        DOCMP      : adr := adr+5+16+16;
        DOCODE     : adr := strobj(adr);
        DOCOL      : result:= COL (adr);
        DOCSTR     : begin WaitForTOTEMPOB:= true; adr := strobj(adr); end;
        DOECMP     : adr := adr+5+5+16+5+16;
        DOEREL     : adr := adr+5+5+16;
        DOEXT      : result:= true;
        DOEXT0     : result:= true;
        DOEXT2     : result:= true;
        DOEXT3     : result:= true;
        DOEXT4     : result:= true;
        DOGROB     : adr := strobj(adr);
        DOHXS      : adr := strobj(adr);
        DOIDNT     : adr := adr+5+2+2*(ReadInteger(adr+5) and $ff);
        DOLAM      : adr := adr+5+2+2*(ReadInteger(adr+5) and $ff);
        DOLIB      : adr := strobj(adr);
        DOLIST     : result:= true;
        DOLNKARRY  : adr := strobj(adr);
        DOREAL     : adr := adr+5+16;
        DOROMP     : adr := adr+5+6;
        DOSYMB     : result:= true;
        DOTAG      : begin result:= false; exit; end;
        // wgg HP49 specific items
        DOFLASHP   : adr := adr+5+7;
        DOINT      : adr := strobj(adr);
        DOLNGCMP   : adr := strobj(adr);
        DOLNGREAL  : adr := strobj(adr);
        DOMATRIX   : result:= true;

        // it is not a prolog.
        // if it is a pointer on a simple object, decompile this object,
        // else, if the pointer has a name, give the name
        // else, give the value of the pointer.
        else
          // skip the pointer
          adr:= adr+5;
          k:=0;
          result:= false;
          while (k<=high(bds)) and not(result) do if bds[k]<>i then inc(k) else result:= true;
      end;
  End;
//var
//  i: integer;
Begin
  TOTEMPOB:= SymbolTable.ValueFromText('TOTEMPOB');
  result:= DecompObj2(adr);
end;

end.
