unit Search;
// wgg ************************************************************
// This is a common search and replace code for the entire system
// TSynEdit for the work is in SrchEditor which is copied
// from frmMainForm.SynAutoComplete1.Editor - always updated when
// we enter an editor box.
//  Copyright 2002, William G. Graves
//******************************************************************
interface

uses
  SynEdit ;

  procedure FindClick(Sender: TObject; var key: word);
  procedure ReplaceClick(Sender: TObject; var key: word);
  procedure NextClick(Sender: TObject; var key: word);
  procedure DoSearchReplaceText(AReplace: boolean; ABackwards: boolean);
  function SetSrchEditor: boolean;
  procedure ShowSearchReplaceDialog(AReplace: boolean);

type
  TSearch=Class(TObject)    // wgg do this just so we can declare a method identifier
  procedure EditorReplaceText(Sender: TObject; const ASearch,
                    AReplace: String; Line, Column: Integer;
                    var Action: TSynReplaceAction);
  end;

implementation
uses
  Windows, Controls, Forms, Classes, FmSourceEdit, registry, FmMainForm,
  dlgSearchText, dlgReplaceText, dlgConfirmReplace, SynCompletionProposal,
  SynEditTypes;

var
  FSearch: TSearch;
  freplKey: string= 'FindReplace';      // registry sub key for the find/replace dialog

  gbSearchBackwards: boolean= false;    // search options - to be saved to the registry
  gbSearchCaseSensitive: boolean= false;
  gbSearchFromCaret: boolean= true;        // wgg influenced by selavail
  gbUserSearchFromCaret: boolean= true;    // wgg user's pref
  gbSearchSelectionOnly: boolean= false;
  gbSearchTextAtCaret: boolean= true;
  gbSearchWholeWords: boolean= false;
  gsSearchText: string;
  gsReplaceText: string;
  
  find: boolean;
  fSearchFromCaret: boolean;
  SrchEditor: TSynEdit;                 // is current editor for search/replace

function SetSrchEditor: boolean;                 // wgg
begin
  SrchEditor:= frmMainForm.SynCompletionProposal1.Editor As TSynEdit;
  if SrchEditor.Height < 42 then result:= false        // wgg minimum of two lines high
  else result:= true;
end;

procedure FindClick(Sender: TObject; var key: word);
begin
  if Not SetSrchEditor then exit;
  find:= True;
  ShowSearchReplaceDialog(False);
  key:= 0;
end;

procedure ReplaceClick(Sender: TObject; var key: word);
begin
  if Not SetSrchEditor then exit;
  Find:= False;
  ShowSearchReplaceDialog(True);
  key:= 0;
end;

procedure NextClick(Sender: TObject; var key: word);
begin
  if Not SetSrchEditor then exit;
  if gbSearchBackwards then
    SrchEditor.BlockEnd := SrchEditor.BlockBegin
  else
    SrchEditor.BlockBegin := SrchEditor.BlockEnd;
  SrchEditor.CaretXY := SrchEditor.BlockBegin;
  DoSearchReplaceText(Not Find, gbSearchBackwards);
  key:= 0;
end;

// wgg this is the beginning of the new search replace code
procedure DoSearchReplaceText(AReplace: boolean;
                          ABackwards: boolean);
var
  Options: TSynSearchOptions;
begin
  SrchEditor.OnReplaceText:= FSearch.EditorReplaceText;

  if AReplace then
    Options := [ssoPrompt, ssoReplace, ssoReplaceAll]
  else
    Options := [];
  if ABackwards then
    Include(Options, ssoBackwards);
  if gbSearchCaseSensitive then
    Include(Options, ssoMatchCase);
  if not fSearchFromCaret then
    Include(Options, ssoEntireScope);
  if gbSearchSelectionOnly then
    Include(Options, ssoSelectedOnly);
  if gbSearchWholeWords then
    Include(Options, ssoWholeWord);
  SrchEditor.SearchEngine := frmMainForm.SynEditSearch1;          // wgg for SynEdit 1.15
  if SrchEditor.SearchReplace(gsSearchText, gsReplaceText, Options) = 0 then
  begin
    MessageBeep(MB_ICONASTERISK);
    if ssoBackwards in Options then
      SrchEditor.BlockEnd := SrchEditor.BlockBegin
    else
      SrchEditor.BlockBegin := SrchEditor.BlockEnd;
    SrchEditor.CaretXY := SrchEditor.BlockBegin;
  end;

  if ConfirmReplaceDialog <> nil then
    ConfirmReplaceDialog.Free;
end;

procedure ShowSearchReplaceDialog(AReplace: boolean);
var
  dlg: TTextSearchDialog;
begin
  if AReplace then
    dlg := TTextReplaceDialog.Create(frmSourceEdit)
  else
    dlg := TTextSearchDialog.Create(frmSourceEdit);
  with dlg do try
    // assign search options
    SearchBackwards:= gbSearchBackwards;
    SearchCaseSensitive:= gbSearchCaseSensitive;
    SearchFromCursor:= gbUserSearchFromCaret;                     // wgg get user pref
    SearchInSelectionOnly:= SrchEditor.SelAvail;                  // wgg was gbSearchSelectionOnly
    // start with last search text
    SearchText := gsSearchText;
    if gbSearchTextAtCaret then begin
      // if something is selected search for that text
      if SrchEditor.SelAvail and (SrchEditor.BlockBegin.Y = SrchEditor.BlockEnd.Y)
      then
        begin
        SearchText := SrchEditor.SelText;
        SearchInSelectionOnly:= false;
      end else
        SearchText := SrchEditor.GetWordAtRowCol(SrchEditor.CaretXY);
    end;
    if SearchInSelectionOnly then SearchFromCursor:= false;       // wgg

    with TRegIniFile.Create(frmSourceEdit.PriKey) do
      try
        SearchTextHistory:= ReadString(freplKey, 'Find', '');
      finally
        free;
      end;

    if AReplace then with dlg as TTextReplaceDialog do begin
      ReplaceText := gsReplaceText;
      with TRegIniFile.Create(frmSourceEdit.PriKey) do
        try
          ReplaceTextHistory:= ReadString(freplKey, 'Replace', '');
        finally
          free;
        end;
    end;
    SearchWholeWords := gbSearchWholeWords;

    if ShowModal = mrOK then begin
      gbSearchBackwards := SearchBackwards;
      gbSearchCaseSensitive := SearchCaseSensitive;
      gbSearchFromCaret := SearchFromCursor;
      gbSearchSelectionOnly := SearchInSelectionOnly;
      gbSearchWholeWords := SearchWholeWords;
      gsSearchText := SearchText;
      if Not gbSearchSelectionOnly then gbUserSearchFromCaret:= SearchFromCursor;     // wgg save user pref
      with TRegIniFile.Create(frmSourceEdit.PriKey) do
        try
          WriteString(freplKey, 'Find', SearchTextHistory);
        finally
          free;
        end;

      if AReplace then with dlg as TTextReplaceDialog do begin
        gsReplaceText := ReplaceText;
        with TRegIniFile.Create(frmSourceEdit.PriKey) do
          try
            WriteString(freplKey, 'Replace', ReplaceTextHistory);
          finally
            free;
          end;
      end;
      fSearchFromCaret := gbSearchFromCaret;
      if gsSearchText <> '' then begin
        DoSearchReplaceText(AReplace, gbSearchBackwards);
        fSearchFromCaret := true;
      end;
    end;
  finally
    if dlg.ModalResult=mrCancel then find:= true;        // wgg do not allow find again to do replace!
    dlg.Free;
  end;
end;

// wgg this procedure is called from the SynEditor OnReplace event
procedure TSearch.EditorReplaceText(Sender: TObject;
          const ASearch, AReplace: String; Line, Column: Integer;
           var Action: TSynReplaceAction);
var
  APos: TPoint;
  EditRect: TRect;
begin
  if ASearch = AReplace then
    Action := raSkip
  else begin
    APos := Point(Column, Line);
    APos := SrchEditor.ClientToScreen(SrchEditor.RowColumnToPixels(APos));
    EditRect := SrchEditor.ClientRect;
    EditRect.TopLeft := SrchEditor.ClientToScreen(EditRect.TopLeft);
    EditRect.BottomRight := SrchEditor.ClientToScreen(EditRect.BottomRight);

    if ConfirmReplaceDialog = nil then
      ConfirmReplaceDialog := TConfirmReplaceDialog.Create(Application);
    ConfirmReplaceDialog.PrepareShow(EditRect, APos.X, APos.Y,
      APos.Y + SrchEditor.LineHeight, ASearch);
    case ConfirmReplaceDialog.ShowModal of
      mrYes: Action := raReplace;
      mrYesToAll: Action := raReplaceAll;
      mrNo: Action := raSkip;
      else Action := raCancel;
    end;
  end;
end;

end.



