{
  This forms allows you to set and clear breakpoints.
  This unit is copyright Heweltt Packard.
}
unit fmBreakPointChoice;

interface

uses
  Classes, Forms, StdCtrls, Spin, Debug, WindowsPosSave, SynEdit, Controls;

type
  TfrmBreakPointChoice = class(TForm)
    Label1: TLabel;
    ListBox1: TListBox;
    btnDelete: TButton;
    btnAdd: TButton;
    chkbRpl: TCheckBox;
    chkbActive: TCheckBox;
    WindowPosSave1: TWindowPosSave;
    gotoEdit1: TSynEdit;
    procedure ListBox1Click(Sender: TObject);
    procedure ListBox1DblClick(Sender: TObject);
    procedure CheckBoxClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure gotoEdit1KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure gotoEdit1KeyPress(Sender: TObject; var Key: Char);
    procedure gotoEdit1Exit(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure gotoEdit1Enter(Sender: TObject);
  private
    Fb: TBreakPoint;
    NoValid: boolean;                    // here to avoid the ValidBreak to fire to often
    procedure Setb(Value: TBreakPoint);  // Change the current viewed breakpoint
    Procedure NewBreak(Sender: TObject); // call back when a new break is created
    Procedure ValidBreak;                // save the value of the current breakpoint
    procedure SetIndex(adrs: integer);   // wgg
  public
    procedure FillListBox;               // recreate the content of the ListBox1 control
    property b: TBreakPoint read Fb write Setb; // the current breakpoint
  end;

var
  frmBreakPointChoice: TfrmBreakPointChoice;

implementation

uses SysUtils, lrWorking, fmSourceEdit, FmMainForm;

{$R *.DFM}

{ TBreakPointChoise }

// click on the list box to select a breakpoint
procedure TfrmBreakPointChoice.ListBox1Click(Sender: TObject);
begin
  b:= debugger.BList[ListBox1.ItemIndex];
end;

// dbl click on a breakpoint to go to the source
procedure TfrmBreakPointChoice.ListBox1DblClick(Sender: TObject);
begin
  Debugger.EditAtAddress(debugger.BList[ListBox1.ItemIndex].Adresse);
end;

// save the current break when click on the 2 check boxes
procedure TfrmBreakPointChoice.CheckBoxClick(Sender: TObject);
begin
  ValidBreak;  
end;


// wgg make the breakpoint whose address is add, the current bp
procedure TfrmBreakPointChoice.SetIndex(adrs: integer);
var
  i: integer;
begin
  For i:=0 to debugger.BreakCount-1 do
    if debugger.BList[i].adresse = adrs then
    begin
      ListBox1.ItemIndex:= i;
      b:= debugger.BList[i];
    end;
end;

// recreate the content of the ListBox1 component, keeping the current break unchanged
procedure TfrmBreakPointChoice.FillListBox;
var
  i, j: integer;
begin
  ListBox1.items.Clear;
  j:= -1;
  For i:=0 to debugger.BreakCount-1 do
  Begin
    ListBox1.items.add(IntToHex(debugger.BList[i].adresse,5) + '  '+SymbolTable.GetFromAdr(debugger.BList[i].adresse));
    if debugger.BList[i] = b then
      j:= i;
    end;
  ListBox1.ItemIndex:= j;
  if j=-1 then
  begin
    Fb:= nil;                                    // wgg changed to Fb, cleared boxes below
    NoValid:= true;                              // dont create until we are done here
    gotoEdit1.text:= '';
    chkbRpl.Checked:= False;
    chkbActive.Checked:= true;
    NoValid:= False;
  end else begin
    NoValid:= true;                              // dont create until we are done here
    gotoEdit1.text:= inttohex(b.Adresse, 6);
    chkbRpl.Checked:= b.RPL;
    chkbActive.Checked:= b.Active;
    NoValid:= False;
  end;
  frmSourceEdit.reloadBP;
end;

// delete the current breakpoint
procedure TfrmBreakPointChoice.btnDeleteClick(Sender: TObject);
var
  adr: integer;
begin
  if b<>nil then
  Begin
    adr:= b.Adresse;
    b:= nil;
    debugger.RemoveBreakPoint(Adr)
  end;
end;

procedure TfrmBreakPointChoice.FormShow(Sender: TObject);
begin
  b:= nil;
end;

// create a new breakpoint.
procedure TfrmBreakPointChoice.btnAddClick(Sender: TObject);
var
  i: integer;
begin
  i:= debugger.GetBreakPointIndex(SymbolTable.ValueFromText(gotoEdit1.Text));
  if i = -1 then  fb:= Nil;     // wgg Nil will force add new bp in ValidBreak
  ValidBreak;                   // else this will edit the existing bp
end;

// change the current breakpoint. First save the current one, Then open the new one
procedure TfrmBreakPointChoice.Setb(Value: TBreakPoint);
begin
  if fb = Value then exit;    // nothing changed, just leave
  ValidBreak;                 // flush current values to the debugger
  fb:= nil;                   // make ready to create
  gotoEdit1.Text:= '';
  btnDelete.Enabled:= value<>nil;
  if value<>nil then
    try
      NoValid:= true;          // dont create until we are done here
      gotoEdit1.text:= inttohex(value.Adresse, 6);
      chkbRpl.Checked:= Value.Rpl;
      chkbActive.Checked:= Value.Active;
      fb:= Value;
    finally
      NoValid:= false;         // now we allow creation
    end;
  FillListBox;
end;

procedure TfrmBreakPointChoice.FormCreate(Sender: TObject);
begin
  Debugger.OnNewBreak:= NewBreak;
  WindowPosSave1.ReadPos;
end;

// recreate the break list when a new break is added
procedure TfrmBreakPointChoice.NewBreak(Sender: TObject);
begin
  FillListBox;
end;

// save the current break (either modify the current break or create a new one)
procedure TfrmBreakPointChoice.ValidBreak;                           // wgg redid a lot of this
var
  i: integer;
  SaveP: PPos;
  userpl: boolean;
begin
  if NoValid then exit;
  if (gotoEdit1.text = '') Or (gotoEdit1.text = #13#10) then exit;
  try
    i:= SymbolTable.ValueFromText(gotoEdit1.Text);
    if (fb<>nil) And (b.adresse=i) then with FB do
    begin
      userpl:= chkbRPL.Checked;       // breakpoint may be outside of Source Code areas
      if i<>-1 then
      begin
        SaveP:= SourceFromAdr(i);
        if SaveP <> Nil then userpl:= SaveP^.isRPL;
        rpl:= userpl;
        Active:= chkbActive.Checked;
        Debugger.RebuiltBreakPointList;
      end;
      FillListBox;
    end else begin
      SaveP:= SourceFromAdr(i);                                    // wgg we need to find out RPL/Asm
      if SaveP <> Nil then userpl:= SaveP^.isRPL
      else userpl:= chkbRPL.Checked;
      Debugger.SetBreakPoint(i, userpl, chkbActive.Checked); // wgg calls FillListBox etc
      SetIndex(i);                                              // wgg make new the current
    end;
  except;
  end;
end;

procedure TfrmBreakPointChoice.gotoEdit1KeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if key=13 then
  Begin
    ValidBreak;
    key:= 0;
  end;
end;

procedure TfrmBreakPointChoice.gotoEdit1KeyPress(Sender: TObject;
  var Key: Char);
begin
  if key=#13 then
    key:= #0;
end;

procedure TfrmBreakPointChoice.gotoEdit1Exit(Sender: TObject);       // wgg exit from edit OR form
begin
  ValidBreak;
end;

procedure TfrmBreakPointChoice.FormActivate(Sender: TObject);
begin
  if ActiveControl is TSynEdit then frmMainForm.EditActivate(Sender);                 // wgg
end;

procedure TfrmBreakPointChoice.gotoEdit1Enter(Sender: TObject);
begin
  if Sender is TSynEdit then frmMainForm.EditEnter(Sender);
end;

end.
